using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;

namespace Sudoku
{
    class DancingLinks
    {
        #region Table, Count, Node, Header; header[], node[], stack, root
        
        SudokuTable t;

        public SudokuTable Table
        {
            get { return t.Clone() as SudokuTable; }
            set { t = value.Clone() as SudokuTable; stack = new Stack(t.M*t.M); }
        }

        int count = 0;

        public int Count
        {
            get { return count; }
        }

        //4 irnyba lncolhat node
        class Node
        {
            public Node left;
            public Node right;
            public Node up;
            public Node down;
            public Header head;
            public int row;

            public Node() : this(-1)
            { }

            public Node(int row)
            {
                left = this;
                right = this;
                up = this;
                down = this;
                this.row = row;
                head = null;
            }
        }
        //node mret, s nv attribtummal
        class Header : Node
        {
            public int size;
            public int name;

            public Header(int name)
            {
                this.name = name;
                this.size = 0;
                this.head = this;
            }
        }

        Header[] columns;   //fejlc, 4*M^2 db (a 4 mdszerhez kln-kln M^2)
        Node[] rows;        //a sorok els elemeit tartalmazza, M^3 db (sorok*oszlok*rtkek)
        Stack stack;        //a sudoku adatai
        Header root;        //az els header-re mutat Header

        #endregion

        public DancingLinks()
        {
            t = new SudokuTable();
            root = new Header(0);   
        }

        public void Init()
        { 
        
        
        }

        //a tbla fejlcnek s sorainak els elemnek ltrehozsa r=M^3, c=4*M^2
		void CreateTable(int r , int c) 
        {
            columns = new Header[c];
            rows = new Node[r];
			//oszlopok fejlceinek ltrehozsa, a fejlc sor vgre lncolsa
			for (int i = 0; i < c; i++)
            {
                columns[i] = new Header(i);
                columns[i].left = root.left;
                columns[i].right = root;
                root.left.right = columns[i];
                root.left = columns[i];
			}
            //sorok els elemeire mutat tmb elemeinek ltrehozsa
            for (int i = 0; i < r; i++)
            {
                rows[i] = new Node();
			}			
		}

		void AppendRow(int[] pos , int row) 
        {
            Node first = new Node(row);
            first.head = columns[pos[0]];
            first.head.size++;
            first.down = columns[pos[0]];
            first.up = columns[pos[0]].up;
            columns[pos[0]].up.down = first;
            columns[pos[0]].up = first;
            rows[row] = first;
			for (int i = 1; i < pos.Length; i++) 
            {
                Node t = new Node(row);
				//befzs sorba
                t.left = first.left;
                t.right = first;
                first.left.right = t;
                first.left = t;
                //befzs a megfelel oszlopba
                t.down = columns[pos[i]];
                t.up = columns[pos[i]].up;
                columns[pos[i]].up.down = t;
                columns[pos[i]].up = t;
				//fejlc bellts
                t.head = columns[pos[i]];
                t.head.size++;
			}
		}

        void BuildTable()
        {
            CreateTable(t.M * t.M * t.M, 4 * t.M * t.M);
            int b = 0;
            for (int i = 0; i < t.M; i++)
            {
                for (int j = 0; j < t.M; j++)
                {
                    b = (i / t.N) * t.N + (j / t.N); //0..M-1
                    for (int d = 0; d < t.M; d++)
                    {
                        AppendRow(new int[] { i * t.M + j, t.M * (t.M + i) + d, t.M * (j + 2 * t.M) + d, t.M * (b + 3 * t.M) + d }, i * t.M * t.M + j * t.M + d);
                    }
                }
            }
        }

		Header FindMin() 
        {
            int min = /*(root.right as Header).size;*/ int.MaxValue;
            Header ret = null;
            for (Header j = (root.right as Header); j != root; j = (j.right as Header))
            {
                if (j.size < min)
                {
                    min = j.size;
                    ret = j;
                }
            }
			return ret;
		}

		void Cover(Header c) 
        { 
			//c header kifzse a header-ek kzl
            c.left.right = c.right;
            c.right.left = c.left;
            //c header oszlopban, illetve azokkal egy sorban lev elemek eltvoltsa
            for (Node i = c.down; i != c; i = i.down)
            {
                for (Node j = i.right; j != i; j = j.right)
                {
                    j.down.up = j.up;
                    j.up.down = j.down;
                    j.head.size--;
                }
            }
		}

		void UnCover(Header c) 
        { 
            for (Node i = c.up; i != c; i = i.up)
            {
                for (Node j = i.left; j != i; j = j.left)
                {
                    j.down.up = j;
                    j.up.down = j;
                    j.head.size++;
                }
            }
            c.left.right = c;
            c.right.left = c;
		}

		public void SelectRow(int index) {
            Node n = rows[index];
			stack.Push(n);
            do
            {
                Cover(n.head);
                n = n.right;
            } while (n != rows[index]);
		}

		public void UnselectRow() 
        {
			Node n = (stack.Pop() as Node).left;
            do
            {
                UnCover(n.head);
                n = n.left;
            } while (n != rows[n.row]);
            UnCover(n.head);
		}

		void UnselectAllRows() 
        {
			while (stack.Count > 0)
            {
				UnselectRow();
			}
		}

        void Solve() 
        {
			if (root.right == root) 
            {
				WriteBackToTable();
                count++;
				return;
			}
			Header c = FindMin(); //a legkisebb mret oszlop
            if (c.size == 0)
            { 
                return; 
            }
			Cover(c);
            Node j = null;
            for (Node r = c.down; r != c; r = r.down)
            {
				stack.Push(r);
                for (j = r.right; j != r; j = j.right)
                {
                    Cover(j.head);
                }
				Solve();
				r = stack.Pop() as Node;
                c = r.head;
                for (j = r.left; j != r; j = j.left)
                {
                    UnCover(j.head);
                }
                if (count > 1) {/* System.Windows.Forms.MessageBox.Show("nem egyrtelm"); */break; }
			}
			UnCover(c);
		}

        //selectrow feltlti adatokkal a ketts lncolt valamit ?
		void SetStartingGrid() 
        {
			for (int i = 0; i < t.M; i++) 
            {
                for (int j = 0; j < t.M; j++) 
                {
					if (t.Cells[i, j].IsData()) 
                    {
                        SelectRow(i * t.M * t.M + j * t.M + (t.Cells[i, j].Data - 1));
					}
				}
			}
		}

        //visszarom a tblba a kiszmolt rtkeket
        void WriteBackToTable()
        {
            int[] solution = new int[stack.Count]; //felveszek a verem mret tmbt
            IEnumerator ien = stack.GetEnumerator();
            int index = 0;
            while (ien.MoveNext())
            {
                solution[index++] = (ien.Current as Node).row;
            }
            Array.Sort(solution);
            for (int i = 0; i < solution.Length; i++)
            {
                solution[i] %= t.M;
            }
            for (int i = 0; i < t.M; i++)
            {
                for (int j = 0; j < t.M; j++)
                {
                    if (t.Cells[i, j].Data != solution[i * t.M + j] + 1)
                    {
                        t.Cells[i, j].Foundby = 1;
                    }
                    t.setCellData(i, j, solution[i * t.M + j] + 1);
                }
            }
        }

        public void Run()
        {
            count = 0;
            root = new Header(0);
            stack.Clear();
            BuildTable();
            SetStartingGrid();
            Solve();
            UnselectAllRows();
        }

        public void Run(System.ComponentModel.BackgroundWorker worker)
        {
            count = 0;
            root = new Header(0);
            stack.Clear();
            BuildTable();
            SetStartingGrid();
            Solve(worker);
            //UnselectAllRows();
        }

        private void Solve(System.ComponentModel.BackgroundWorker worker)
        {
            if (root.right == root)
            {
                WriteBackToTable();
                count++;
                return;
            }
            Header c = FindMin(); //a legkisebb mret oszlop
            if (c.size == 0)
            {
                return;
            }
            Cover(c);
            Node j = null;
            for (Node r = c.down; r != c; r = r.down)
            {
                stack.Push(r);
                for (j = r.right; j != r; j = j.right)
                {
                    Cover(j.head);
                }
                if (worker.CancellationPending)
                {
                    //System.Windows.Forms.MessageBox.Show("fd");
                    break;
                }
                Solve(worker);
                if (worker.CancellationPending)
                {
                    //System.Windows.Forms.MessageBox.Show("fd");
                    break;
                }
                r = stack.Pop() as Node;
                c = r.head;
                for (j = r.left; j != r; j = j.left)
                {
                    UnCover(j.head);
                }
                if (count > 1) { /* System.Windows.Forms.MessageBox.Show("nem egyrtelm"); */break; }
            }
            if (worker.CancellationPending)
            {
                //System.Windows.Forms.MessageBox.Show("fd");
                return;
            }
            UnCover(c);
        }
    }
}
