using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Sudoku
{
    class HiddenSingle : Algorithm
    {
        enum house{row, column, box};
        bool[,] patterns;
        class haromelem
        {
            public haromelem(bool row, bool column, bool box)
            {
                this.row = row;
                this.column = column;
                this.box = box;
            }
        
            public bool row;
            public bool column;
            public bool box;
        }

        public HiddenSingle()
        {
            name = "Hidden Single";
        }

        string globmessage = string.Empty;

        private void HiddenToQueueByRow(int c, int i, ref string hit)
        {
            int sum = 0;
            int col = 0;
            bool found = false;
            for (int j = 0; (j < t.M) && (sum < 2); j++)
            {
                if (t.Cells[i, j].Data == c)
                {
                    found = true;
                }
                if (t.Cells[i, j].IsCandidate(c) && !t.Cells[i, j].IsData())
                {
                    sum++;
                    col = j;
                }
            }

            if (sum == 0 && !found)
            {
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(new HiddenErrorPattern(new IntPair(i, 0), i, RectangleTypes.Row, c));
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(new HiddenErrorPattern(new IntPair(i, 0), i, RectangleTypes.Row, c));
                }
            }

            if (sum == 1 && (patterns[i, col] == false || AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve))
            {
                List<int> candidates = new List<int>();
                List<IntPair> candidatePositions = new List<IntPair>();
                for (int k = 1; k <= t.M; ++k)
                {
                    if (k != c && t.Cells[i, col].IsCandidate(k))
                    {
                        candidatePositions.Add(new IntPair(i, col));
                        candidates.Add(k);
                    }
                }

                for (int k = 0; k < t.M; ++k)
                {
                    if (k != i && !t.Cells[k, col].IsData() && t.Cells[k, col].IsCandidate(c))
                    {
                        candidatePositions.Add(new IntPair(k, col));
                        candidates.Add(c);
                    }
                }

                for (int xi = (i / t.N) * t.N; i / t.N == xi / t.N; ++xi)
                {
                    for (int xj = (col / t.N) * t.N; col / t.N == xj / t.N; ++xj)
                    {
                        if (xi != i && !t.Cells[xi, xj].IsData() && t.Cells[xi, xj].IsCandidate(c))
                        {
                            candidatePositions.Add(new IntPair(xi, xj));
                            candidates.Add(c);
                        }
                    }
                }

                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                gc.Add(c);
                gcc.Add(new IntPair(i, col));
                HiddenSinglePattern p = new HiddenSinglePattern(c, gc, gcc, candidates, candidatePositions, RectangleTypes.Row);

                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                    AlgorithmManager.Instance.ApplyPattern(p);
                }
                else if (AlgorithmManager.Instance.SolveState != SolveStates.WritingData)
                {
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                    AlgorithmManager.Instance.AddPattern(p);
                }
                patterns[i, col] = true;
            }
        }

        private void HiddenToQueueByColumn(int c, int j, ref string hit)
        {
            int sum = 0;
            int row = 0;
            bool found = false;
            for (int i = 0; (i < t.M) && (sum < 2); i++)
            {
                if (t.Cells[i, j].Data == c)
                {
                    found = true;
                }
                if (t.Cells[i, j].IsCandidate(c) && !t.Cells[i, j].IsData())
                {
                    sum++;
                    row = i;
                }
            }

            if (sum == 0 && !found/* && AlgorithmManager.Instance.SolveState == SolveStates.UserSolve*/)
            {
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(new HiddenErrorPattern(new IntPair(0, j), j, RectangleTypes.Column, c));
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(new HiddenErrorPattern(new IntPair(0, j), j, RectangleTypes.Column, c));
                }
            }

            if (sum == 1 && (patterns[row, j] == false || AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve))
            {
                List<int> candidates = new List<int>();
                List<IntPair> candidatePositions = new List<IntPair>();
                for (int k = 1; k <= t.M; ++k)
                {
                    if (k != c && !t.Cells[row, j].IsData() && t.Cells[row, j].IsCandidate(k))
                    {
                        candidatePositions.Add(new IntPair(row, j));
                        candidates.Add(k);
                    }
                }
                for (int k = 0; k < t.M; ++k)
                {
                    if (k != j && !t.Cells[row, k].IsData() && t.Cells[row, k].IsCandidate(c))
                    {
                        candidatePositions.Add(new IntPair(row, k));
                        candidates.Add(c);
                    }
                }

                for (int xi = (row / t.N) * t.N; row / t.N == xi / t.N; ++xi)
                {
                    for (int xj = (j / t.N) * t.N; j / t.N == xj / t.N; ++xj)
                    {
                        if (xj != j && xi != row && !t.Cells[xi, xj].IsData() && t.Cells[xi, xj].IsCandidate(c))
                        {
                            candidatePositions.Add(new IntPair(xi, xj));
                            candidates.Add(c);
                        }
                    }
                }

                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                gc.Add(c);
                gcc.Add(new IntPair(row, j));

                HiddenSinglePattern p = new HiddenSinglePattern(c, gc, gcc, candidates, candidatePositions, RectangleTypes.Column);
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                    AlgorithmManager.Instance.ApplyPattern(p);
                }
                else if (AlgorithmManager.Instance.SolveState != SolveStates.WritingData)
                {
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                    AlgorithmManager.Instance.AddPattern(p);
                }

                patterns[row, j] = true;
            }
        }

        private void HiddenToQueueByBlock(int c, int k, int l, ref string hit)
        {
            int sum = 0;
            int row = 0;
            int col = 0;
            int bx = k * t.N;
            int by = l * t.N;
            bool found = false;
            for (int i = bx; i < bx + t.N && (sum < 2); i++)
            {
                for (int j = by; j < by + t.N && (sum < 2); j++)
                {
                    if (t.Cells[i, j].Data == c)
                    {
                        found = true;
                    }
                    if (t.Cells[i, j].IsCandidate(c) && !t.Cells[i, j].IsData())
                    {
                        sum++;
                        col = j;
                        row = i;
                    }
                }
            }

            if (sum == 0 && !found/* && AlgorithmManager.Instance.SolveState == SolveStates.UserSolve*/)
            {
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(new HiddenErrorPattern(new IntPair(k * t.N, l * t.N), k * t.N + l + 1, RectangleTypes.Box, c));
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(new HiddenErrorPattern(new IntPair(k * t.N, l * t.N), k * t.N + l + 1, RectangleTypes.Box, c));
                }
            }

            if (sum == 1 && (patterns[row, col] == false || AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve))
            {
                List<int> candidates = new List<int>();
                List<IntPair> candidatePositions = new List<IntPair>();
                for (int w = 1; w <= t.M; ++w)
                {
                    if (w != c && t.Cells[row, col].IsCandidate(w))
                    {
                        candidatePositions.Add(new IntPair(row, col));
                        candidates.Add(w);
                    }
                }

                for (int w = 0; w < t.M; ++w)
                {
                    if (w != col && !t.Cells[row, w].IsData() && t.Cells[row, w].IsCandidate(c))
                    {
                        candidatePositions.Add(new IntPair(row, w));
                        candidates.Add(c);
                    }
                }

                for (int w = 0; w < t.M; ++w)
                {
                    if (w != row && !t.Cells[w, col].IsData() && t.Cells[w, col].IsCandidate(c))
                    {
                        candidatePositions.Add(new IntPair(w, col));
                        candidates.Add(c);
                    }
                }

                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                gc.Add(c);
                gcc.Add(new IntPair(row, col));

                HiddenSinglePattern p = new HiddenSinglePattern(c, gc, gcc, candidates, candidatePositions, RectangleTypes.Box);
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(p);
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                }
                else if (AlgorithmManager.Instance.SolveState != SolveStates.WritingData)
                {
                    AlgorithmManager.Instance.AddPattern(p);
                    hit += "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                }

                patterns[row, col] = true;
            }
        }

        public override string TestCell(SudokuTable table, IntPair redCell, IntPair greenCell, int redCand, int greenCand)
        {
            if (!Enabled) return "";

            t = table;
            if (patterns == null) patterns = new bool[t.M, t.M];
            string foundNewPatternString = "";

            if (redCell.i != greenCell.i || (redCell.i == greenCell.i && redCell.j == greenCell.j))
            {
                HiddenToQueueByRow(redCand, redCell.i, ref foundNewPatternString);
            }
            if (redCell.j != greenCell.j || (redCell.i == greenCell.i && redCell.j == greenCell.j))
            {
                HiddenToQueueByColumn(redCand, redCell.j, ref foundNewPatternString);
            }
            if ((!(redCell.i / t.N == greenCell.i / t.N && redCell.j / t.N == greenCell.j / t.N) || redCand != greenCand))
            {
                HiddenToQueueByBlock(redCand, redCell.i / t.N, redCell.j / t.N, ref foundNewPatternString);
            }

            return foundNewPatternString;
        }

        public override void solve(SudokuTable table)
        {
            if (!this.Enabled)
            {
                return;
            }

            t = table;
            string junk = "";
            patterns = new bool[t.M, t.M];

            for (int i = 0; i < t.M; i++)
            {
                for (int c = 1; c <= t.M; c++)
                {
                    HiddenToQueueByRow(c, i, ref junk);
                }
            }

            for (int j = 0; j < t.M; j++)
            {
                for (int c = 1; c <= t.M; c++)
                {
                    HiddenToQueueByColumn(c, j, ref junk);
                }
            }

            for (int k = 0; k < t.N; k++)
            {
                for (int l = 0; l < t.N; l++)
                {
                    for (int c = 1; c <= t.M; c++)
                    {
                        HiddenToQueueByBlock(c, k, l, ref junk);
                    }
                }
            }
        }
    }
}
