﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    class HiddenTriple : Algorithm
    {
        public HiddenTriple()
        {
            name = "Hidden Triple";
        }

        string globmessage = string.Empty;

        public override string TestCell(SudokuTable table, IntPair redCell, IntPair greenCell, int redCand, int greenCand)
        {
            if (!Enabled) return "";

            t = table;
            SearchHiddenTripleInRow(redCell.i);
            SearchHiddenTripleInColumn(redCell.j);
            SearchHiddenTripleInBox(redCell.i / t.N, redCell.j / t.N);

            return "";
        }

        private void SearchHiddenTripleInRow(int row)
        {
            int[] candidate_numbers = new int[t.M + 1];
            bool[,] candidate_positions = new bool[t.M + 1, t.M];

            //a jeloltek elofordulasinak es elofordulasok szamanak kigyujtese
            for (int j = 0; j < t.M; ++j)
            {
                if (!t.Cells[row, j].IsData())
                {
                    for (int c = 1; c <= t.M; ++c)
                    {
                        if (t.Cells[row, j].IsCandidate(c))
                        {
                            candidate_positions[c, j] = true;
                            ++candidate_numbers[c];
                        }
                    }
                }
            }

            //a kigyujtott tombbot bejarni harom indexel
            for (int c1 = 1; c1 <= t.M - 2; ++c1)
            {
                for (int c2 = c1 + 1; c2 <= t.M - 1; ++c2)
                {
                    for (int c3 = c2 + 1; c3 <= t.M; ++c3)
                    {
                        //megfelelo szamu elofordulas van-e
                        if (candidate_numbers[c1] >= 2 && candidate_numbers[c1] <= 3 && candidate_numbers[c2] >= 2 &&
                            candidate_numbers[c2] <= 3 && candidate_numbers[c3] >= 2 && candidate_numbers[c3] <= 3)
                        {
                            //elofordulasok ugyanabban a 3 cellaban vannak-e
                            IntPair[] cells = new IntPair[3];
                            int count = 0;

                            for (int k = 0; k < t.M; ++k)
                            {

                                if (candidate_positions[c1, k] == true || candidate_positions[c2, k] == true || candidate_positions[c3, k] == true)
                                {
                                    if (count < 3)
                                    {
                                        cells[count] = new IntPair(row, k);
                                    }
                                    ++count;
                                }
                            }

                            //ha igen
                            if (count == 3)
                            {
                                //vannak-e kizarhato jeloltek
                                int coun = 0;
                                for (int n = 1; n <= t.M; ++n)
                                {
                                    if (t.Cells[cells[0].i, cells[0].j].IsCandidate(n) || t.Cells[cells[1].i, cells[1].j].IsCandidate(n) ||
                                        t.Cells[cells[2].i, cells[2].j].IsCandidate(n))
                                    {
                                        ++coun;
                                    }
                                }

                                //ha igen minta hozzaadasa
                                if (coun > 3)
                                {
                                    List<IntPair> rcc = new List<IntPair>();
                                    List<int> rc = new List<int>();
                                    List<IntPair> gcc = new List<IntPair>();
                                    List<int> gc = new List<int>();

                                    //a mintaban szereplo jeloltek megkesesese
                                    for (int c = 1; c <= t.M; ++c)
                                    {
                                        for (int i = 0; i < 3; ++i)
                                        {
                                            if (c == c1 || c == c2 || c == c3)
                                            {
                                                gcc.Add(cells[i]);
                                                gc.Add(c);
                                            }
                                            else if (t.Cells[cells[i].i, cells[i].j].IsCandidate(c))
                                            {
                                                rcc.Add(cells[i]);
                                                rc.Add(c);
                                            }
                                        }
                                    }

                                    HiddenTriplePattern p = new HiddenTriplePattern(0, gc, gcc, rc, rcc);
                                    p.HouseType = RectangleTypes.Row;
                                    if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                                    {
                                        AlgorithmManager.Instance.ApplyPattern(p);
                                    }
                                    else
                                    {
                                        AlgorithmManager.Instance.AddPattern(p);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void SearchHiddenTripleInColumn(int col)
        {
            int[] candidate_numbers = new int[t.M + 1];
            bool[,] candidate_positions = new bool[t.M + 1, t.M];

            //a jeloltek elofordulasinak es elofordulasok szamanak kigyujtese
            for (int i = 0; i < t.M; ++i)
            {
                if (!t.Cells[i, col].IsData())
                {
                    for (int c = 1; c <= t.M; ++c)
                    {
                        if (t.Cells[i, col].IsCandidate(c))
                        {
                            candidate_positions[c, i] = true;
                            ++candidate_numbers[c];
                        }
                    }
                }
            }

            //a kigyujtott tombbot bejarni harom indexel
            for (int c1 = 1; c1 <= t.M - 2; ++c1)
            {
                for (int c2 = c1 + 1; c2 <= t.M - 1; ++c2)
                {
                    for (int c3 = c2 + 1; c3 <= t.M; ++c3)
                    {
                        //megfelelo szamu elofordulas van-e
                        if (candidate_numbers[c1] >= 2 && candidate_numbers[c1] <= 3 && candidate_numbers[c2] >= 2 &&
                            candidate_numbers[c2] <= 3 && candidate_numbers[c3] >= 2 && candidate_numbers[c3] <= 3)
                        {
                            //elofordulasok ugyanabban a 3 cellaban vannak-e
                            IntPair[] cells = new IntPair[3];
                            int count = 0;

                            for (int k = 0; k < t.M; ++k)
                            {

                                if (candidate_positions[c1, k] == true || candidate_positions[c2, k] == true || candidate_positions[c3, k] == true)
                                {
                                    if (count < 3)
                                    {
                                        cells[count] = new IntPair(k, col);
                                    }
                                    ++count;
                                }
                            }

                            //ha igen
                            if (count == 3)
                            {
                                //vannak-e kizarhato jeloltek
                                int coun = 0;
                                for (int n = 1; n <= t.M; ++n)
                                {
                                    if (t.Cells[cells[0].i, cells[0].j].IsCandidate(n) || t.Cells[cells[1].i, cells[1].j].IsCandidate(n) ||
                                        t.Cells[cells[2].i, cells[2].j].IsCandidate(n))
                                    {
                                        ++coun;
                                    }
                                }

                                //ha igen minta hozzaadasa
                                if (coun > 3)
                                {
                                    List<IntPair> rcc = new List<IntPair>();
                                    List<int> rc = new List<int>();
                                    List<IntPair> gcc = new List<IntPair>();
                                    List<int> gc = new List<int>();

                                    for (int c = 1; c <= t.M; ++c)
                                    {
                                        for (int i = 0; i < 3; ++i)
                                        {
                                            if (c == c1 || c == c2 || c == c3)
                                            {
                                                gcc.Add(cells[i]);
                                                gc.Add(c);
                                            }
                                            else if (t.Cells[cells[i].i, cells[i].j].IsCandidate(c))
                                            {
                                                rcc.Add(cells[i]);
                                                rc.Add(c);
                                            }
                                        }
                                    }

                                    HiddenTriplePattern p = new HiddenTriplePattern(0, gc, gcc, rc, rcc);
                                    p.HouseType = RectangleTypes.Column;
                                    if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                                    {
                                        AlgorithmManager.Instance.ApplyPattern(p);
                                    }
                                    else
                                    {
                                        AlgorithmManager.Instance.AddPattern(p);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void SearchHiddenTripleInBox(int Xbox, int Ybox)
        {
            int[] candidate_numbers = new int[t.M + 1];
            bool[,] candidate_positions = new bool[t.M + 1, t.M];

            int start_X = Xbox * t.N;
            int start_Y = Ybox * t.N;

            for (int i = start_X; i < start_X + t.N; ++i)
            {
                for (int j = start_Y; j < start_Y + t.N; ++j)
                {
                    if (!t.Cells[i, j].IsData())
                    {
                        for (int c = 1; c < t.M; ++c)
                        {
                            if (t.Cells[i, j].IsCandidate(c))
                            {
                                candidate_positions[c, (j - start_Y) * t.N + (i - start_X)] = true;
                                ++candidate_numbers[c];
                            }
                        }
                    }
                }
            }

            //a kigyujtott tombbot bejarni harom indexel
            for (int c1 = 1; c1 <= t.M - 2; ++c1)
            {
                for (int c2 = c1 + 1; c2 <= t.M - 1; ++c2)
                {
                    for (int c3 = c2 + 1; c3 <= t.M; ++c3)
                    {
                        //megfelelo szamu elofordulas van-e
                        if (candidate_numbers[c1] >= 2 && candidate_numbers[c1] <= 3 && candidate_numbers[c2] >= 2 &&
                            candidate_numbers[c2] <= 3 && candidate_numbers[c3] >= 2 && candidate_numbers[c3] <= 3)
                        {
                            //elofordulasok ugyanabban a 3 cellaban vannak-e
                            IntPair[] cells = new IntPair[3];
                            int count = 0;

                            for (int k = 0; k < t.M; ++k)
                            {
                                if (candidate_positions[c1, k] == true || candidate_positions[c2, k] == true || candidate_positions[c3, k] == true)
                                {
                                    if (count < 3)
                                    {
                                        cells[count] = new IntPair(start_X + k % t.N, start_Y + k / t.N);
                                    }
                                    ++count;
                                }
                            }

                            //ha igen
                            if (count == 3)
                            {
                                //vannak-e kizarhato jeloltek
                                int coun = 0;
                                for (int k = 1; k <= t.M; ++k)
                                {
                                    if (t.Cells[cells[0].i, cells[0].j].IsCandidate(k) || t.Cells[cells[1].i, cells[1].j].IsCandidate(k) ||
                                        t.Cells[cells[2].i, cells[2].j].IsCandidate(k))
                                    {
                                        ++coun;
                                    }
                                }

                                //ha igen minta hozzaadasa
                                if (coun > 3)
                                {
                                    List<IntPair> rcc = new List<IntPair>();
                                    List<int> rc = new List<int>();
                                    List<IntPair> gcc = new List<IntPair>();
                                    List<int> gc = new List<int>();

                                    for (int c = 1; c <= t.M; ++c)
                                    {
                                        for (int i = 0; i < 3; ++i)
                                        {
                                            if (c == c1 || c == c2 || c == c3)
                                            {
                                                gcc.Add(cells[i]);
                                                gc.Add(c);
                                            }
                                            else if (t.Cells[cells[i].i, cells[i].j].IsCandidate(c))
                                            {
                                                rcc.Add(cells[i]);
                                                rc.Add(c);
                                            }
                                        }
                                    }

                                    HiddenTriplePattern p = new HiddenTriplePattern(0, gc, gcc, rc, rcc);
                                    p.HouseType = RectangleTypes.Box;
                                    if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                                    {
                                        AlgorithmManager.Instance.ApplyPattern(p);
                                    }
                                    else
                                    {
                                        AlgorithmManager.Instance.AddPattern(p);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        public override void solve(SudokuTable table)
        {
            t = table;
            if (!this.Enabled)
            {
                return;
            }

            ////////SOR
            for (int row = 0; row < t.M; ++row)
            {
                SearchHiddenTripleInRow(row);
            }

            ////////OSZLOP
            for (int col = 0; col < t.M; ++col)
            {
                SearchHiddenTripleInColumn(col);
            }

            ////////BLOKK
            for (int Xbox = 0; Xbox < t.N; ++Xbox)
            {
                for (int Ybox = 0; Ybox < t.N; ++Ybox)
                {
                    SearchHiddenTripleInBox(Xbox, Ybox);
                }
            }
        }
    }
}