using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    class NakedPair : Algorithm
    {
        string globmessage = string.Empty;

        public NakedPair()
        {
            name = "Naked Pair";
        }

        public override void solve(SudokuTable table)
        {
            if (!this.Enabled)
            {
                return;
            }
            t = table;

            ////////SOR
            for (int r = 0; r < t.M; ++r)
            {
                TestCellInRow(r);
            }

            ////////OSZLOP
            for (int c = 0; c < t.M; ++c)
            {
                TestCellInColumn(c);
            }

            ////////BLOKK
            //vegig a blokkokon
            for (int bi = 0; bi < t.N; ++bi)
            {
                for (int bj = 0; bj < t.N; ++bj)
                {
                    TestCellInBox(bi, bj);
                }
            }
        }

        public override string TestCell(SudokuTable table, IntPair redCell, IntPair greenCell, int redCand, int greenCand)
        {
            if (!Enabled) return "";
            t = table;
            if (t.Cells[redCell.i, redCell.j].CandidatesNumber == 2)
            {
                TestCellInRow(redCell.i);
                TestCellInColumn(redCell.j);
                TestCellInBox(redCell.i / t.N, redCell.j / t.N);
            }

            return "";
        }

        private void TestCellInRow(int row)
        {
            List<IntPair> cells = new List<IntPair>();
            for (int j = 0; j < t.M; ++j)
            {
                if (!t.Cells[row, j].IsData() && t.Cells[row, j].CandidatesNumber == 2)
                {
                    cells.Add(new IntPair(row, j));
                }
            }

            for (int p1 = 0; p1 < cells.Count; ++p1)
            {
                for (int p2 = p1 + 1; p2 < cells.Count; ++p2)
                {
                    int sum = 0;
                    bool[] test = new bool[t.M + 1];
                    for (int cand = 1; cand <= t.M; ++cand)
                    {
                        if (t.Cells[cells[p1].i, cells[p1].j].IsCandidate(cand) || t.Cells[cells[p2].i, cells[p2].j].IsCandidate(cand))
                        {
                            test[cand] = true;
                            ++sum;
                        }
                    }

                    if (sum == 2)
                    {
                        CreatePatternRow(test, row, cells[p1].j, cells[p2].j);
                    }
                }
            }
        }

        private void TestCellInColumn(int col)
        {
            List<IntPair> cells = new List<IntPair>();
            for (int i = 0; i < t.M; ++i)
            {
                if (!t.Cells[i, col].IsData() && t.Cells[i, col].CandidatesNumber == 2)
                {
                    cells.Add(new IntPair(i, col));
                }
            }

            for (int p1 = 0; p1 < cells.Count; ++p1)
            {
                for (int p2 = p1 + 1; p2 < cells.Count; ++p2)
                {
                    int sum = 0;
                    bool[] test = new bool[t.M + 1];
                    for (int ind = 1; ind <= t.M; ++ind)
                    {
                        if (t.Cells[cells[p1].i, cells[p1].j].IsCandidate(ind) || t.Cells[cells[p2].i, cells[p2].j].IsCandidate(ind))
                        {
                            test[ind] = true;
                            ++sum;
                        }
                    }

                    if (sum == 2)
                    {
                        CreatePatternColumn(test, col, cells[p1].i, cells[p2].i);
                    }
                }
            }
        }

        private void TestCellInBox(int bi, int bj)
        {
            //Azok a cellak, amik szerepelhetnek tiszta par mintaban
            //Tovabbiakban kenyelmesebb ezt vizsgalni
            List<IntPair> cells = new List<IntPair>();
            for (int i = bi * t.N; i / t.N == bi; ++i)
            {
                for (int j = bj * t.N; j / t.N == bj; ++j)
                {
                    if (!t.Cells[i, j].IsData() && t.Cells[i, j].CandidatesNumber == 2)
                    {
                        cells.Add(new IntPair(i, j));
                    }
                }
            }

            for (int p1 = 0; p1 < cells.Count; ++p1)
            {
                for (int p2 = p1 + 1; p2 < cells.Count; ++p2)
                {
                    int sum = 0;
                    bool[] test = new bool[t.M + 1];
                    for (int i = 1; i <= t.M; ++i)
                    {
                        if (t.Cells[cells[p1].i, cells[p1].j].IsCandidate(i) || t.Cells[cells[p2].i, cells[p2].j].IsCandidate(i))
                        {
                            test[i] = true;
                            ++sum;
                        }
                    }

                    if (sum == 2)
                    {
                        CreatePatternBlock(test, cells[p1].i, cells[p1].j, cells[p2].i, cells[p2].j);
                    }
                }
            }
        }

        private void CreatePatternRow(bool[] test, int r, int c1, int c2)
        {
            List<IntPair> rcc = new List<IntPair>();
            List<int> rc = new List<int>();
            int[] cand = new int[2];

            //jelltek kigyjtse
            int i = 0;
            for (int c = 1; c <= t.M; c++)
            {
                if (test[c])
                {
                    cand[i++] = c;
                }
            }

            int sum = 0; //ezzel a tallattal hny helyrl hzhatok

            //vgig a soron
            for (int l = 0; l < t.M; l++)
            {
                //ha hzhat, teht i, l nem eleme match-nek 
                if (!(l == c1 || l == c2))
                {
                    //minden jellt
                    for (int jj = 0; jj < 2; jj++)
                    {
                        //ha jellt s nincs adat, akkor hzsra kijells
                        if (t.Cells[r, l].IsCandidate(cand[jj]) && !t.Cells[r, l].IsData())
                        {
                            sum++;
                            rcc.Add(new IntPair(r, l)); //honnan
                            rc.Add(cand[jj]); //mit
                        }
                    }
                }
            }
            //ha tudtam hzni
            if (sum != 0)
            {
                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                for (i = 0; i < 2; ++i)
                {
                    gcc.Add(new IntPair(r, c1));
                    gc.Add(cand[i]);
                    gcc.Add(new IntPair(r, c2));
                    gc.Add(cand[i]);
                }

                NakedPairPattern p = new NakedPairPattern(0, gc, gcc, rc, rcc);
                p.HouseType = RectangleTypes.Row;

                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(p);
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(p);
                }
            }
        }

        private void CreatePatternColumn(bool[] test, int c, int c1, int c2)
        {
            List<IntPair> q = new List<IntPair>();
            List<int> cq = new List<int>();
            int[] cand = new int[2];

            //jelltek kigyjtse
            int i = 0;
            for (int p = 1; p <= t.M; p++)
            {
                if (test[p])
                {
                    cand[i++] = p;
                }
            }

            int sum = 0; //ezzel a tallattal hny helyrl hzhatok
            //vgig az oszlopon
            for (int l = 0; l < t.M; l++)
            {
                //ha hzhat, teht l, j nem eleme match-nek 
                if (!(l == c1 || l == c2))
                {
                    //minden jellt
                    for (int jj = 0; jj < 2; jj++)
                    {
                        //ha jellt s nincs adat, akkor hzsra kijells
                        if (t.Cells[l, c].IsCandidate(cand[jj]) && !t.Cells[l, c].IsData())
                        {
                            sum++;
                            q.Add(new IntPair(l, c)); //honnan
                            cq.Add(cand[jj]); //mit
                        }
                    }
                }
            }
            //ha tudtam hzni
            if (sum != 0)
            {
                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                for (i = 0; i < 2; ++i)
                {
                    gcc.Add(new IntPair(c1, c));
                    gc.Add(cand[i]);
                    gcc.Add(new IntPair(c2, c));
                    gc.Add(cand[i]);
                }

                NakedPairPattern p = new NakedPairPattern(0, gc, gcc, cq, q);
                p.HouseType = RectangleTypes.Column;

                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(p);
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(p);
                }
            }
        }

        private void CreatePatternBlock(bool[] test, int ci1, int cj1, int ci2, int cj2)
        {
            List<IntPair> q = new List<IntPair>();
            List<int> cq = new List<int>();
            int[] cand = new int[2];

            //aktulis blokk bal fels indexe
            int bx = (ci1 / t.N) * t.N;
            int by = (cj1 / t.N) * t.N;

            //jelltek kigyjtse
            int i = 0;
            for (int c = 1; c <= t.M; c++)
            {
                if (test[c])
                {
                    cand[i++] = c;
                }
            }

            int sum = 0; //ezzel a tallattal hny helyrl hzhatok

            for (int kk = bx; kk < bx + t.N; kk++)
            {
                for (int ll = by; ll < by + t.N; ll++)
                {
                    //ha hzhat, teht kk,ll nem eleme match-nek 
                    if (!((kk == ci1 && ll == cj1) || (kk == ci2 && ll == cj2)))
                    {
                        //minden jellt
                        for (int jj = 0; jj < 2; jj++)
                        {
                            //ha jellt s nincs adat, akkor hzsra kijells
                            if (t.Cells[kk, ll].IsCandidate(cand[jj]) && !t.Cells[kk, ll].IsData())
                            {
                                sum++;
                                q.Add(new IntPair(kk, ll)); //honnan
                                cq.Add(cand[jj]); //mit
                            }
                        }
                    }
                }
            }

            //ha tudtam hzni
            if (sum != 0)
            {
                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                for (i = 0; i < 2; ++i)
                {
                    gcc.Add(new IntPair(ci1, cj1));
                    gc.Add(cand[i]);
                    gcc.Add(new IntPair(ci2, cj2));
                    gc.Add(cand[i]);
                }

                NakedPairPattern p = new NakedPairPattern(0, gc, gcc, cq, q);
                p.HouseType = RectangleTypes.Box;

                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(p);
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(p);
                }
            }

        }
    }
}
