using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    class NakedSingle : Algorithm
    {
        string globmessage = string.Empty;

        public NakedSingle()
        {
            name = "Naked Single";
        }

        public override string TestCell(SudokuTable table, IntPair redCell, IntPair greenCell, int redCand, int greenCand)
        {
            if (!Enabled || (redCell.i == greenCell.i && redCell.j == greenCell.j)) return "";

            t = table;

            if (t.Cells[redCell.i, redCell.j].CandidatesNumber == 0 && !t.Cells[redCell.i, redCell.j].IsData())
            {
                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(new NakedErrorPattern(redCell));
                }
                else
                {
                    AlgorithmManager.Instance.AddPattern(new NakedErrorPattern(redCell));
                }
                return "";
            }

            if ((t.Cells[redCell.i, redCell.j].CandidatesNumber == 1) && !t.Cells[redCell.i, redCell.j].IsData())
            {
                for (int k = 1; k <= t.M; ++k)
                {
                    if (t.Cells[redCell.i, redCell.j].IsCandidate(k))
                    {
                        redCand = k;
                    }
                }

                List<IntPair> candidatePositions = new List<IntPair>();
                List<int> candidates = new List<int>();

                for (int k = 0; k < t.M; ++k)
                {
                    if (k != redCell.j && !t.Cells[redCell.i, k].IsData() && t.Cells[redCell.i, k].IsCandidate(redCand))
                    {
                        candidatePositions.Add(new IntPair(redCell.i, k));
                        candidates.Add(redCand);
                    }
                }

                for (int k = 0; k < t.M; ++k)
                {
                    if (k != redCell.i && !t.Cells[k, redCell.j].IsData() && t.Cells[k, redCell.j].IsCandidate(redCand))
                    {
                        candidatePositions.Add(new IntPair(k, redCell.j));
                        candidates.Add(redCand);
                    }
                }

                for (int xi = (redCell.i / t.N) * t.N; redCell.i / t.N == xi / t.N; ++xi)
                {
                    for (int xj = (redCell.j / t.N) * t.N; redCell.j / t.N == xj / t.N; ++xj)
                    {
                        if (xi != redCell.i && xj != redCell.j && !t.Cells[xi, xj].IsData() && t.Cells[xi, xj].IsCandidate(redCand))
                        {
                            candidatePositions.Add(new IntPair(xi, xj));
                            candidates.Add(redCand);
                        }
                    }
                }

                List<IntPair> gcc = new List<IntPair>();
                List<int> gc = new List<int>();
                gc.Add(redCand);
                gcc.Add(new IntPair(redCell.i, redCell.j));
                Pattern p = new NakedSinglePattern(redCand, gc, gcc, candidates, candidatePositions);

                if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve)
                {
                    AlgorithmManager.Instance.ApplyPattern(p);
                    return "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                }
                else if (AlgorithmManager.Instance.SolveState != SolveStates.WritingData)
                {
                    AlgorithmManager.Instance.AddPattern(p);
                    return "\\par" + Environment.NewLine + "  \\tabs\\tab -> " + p.Text;
                }

                return "";
            }
            
            if (!t.Cells[redCell.i, redCell.j].IsData() && t.Cells[redCell.i, redCell.j].CandidatesNumber == 0)
            {
                //if (AlgorithmManager.Instance.SolveState == SolveStates.UserSolve)
                AlgorithmManager.Instance.AddPattern(new NakedErrorPattern(new IntPair(redCell.i, redCell.j)));
            }

            return "";
        }

        public override void solve(SudokuTable table)
        {
            t = table;
            if (!this.Enabled)
            {
                return;
            }

            for (int i = 0; i < t.M; i++)
            {
                for (int j = 0; j < t.M; j++)
                {
                    TestCell(this.t, new IntPair(i, j), new IntPair(-1, -1), 0, 0);
                }
            }
        }
    }
}
