﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    class Nishio
    {
        private class GuessOption
        {
            public int Candidate;
            public IntPair CandidateCell;
        }

        private SudokuTable backup;
        private GuessOption currentTipp;
        private bool initialized;

        Queue<GuessOption> guessOptions;

        public Nishio()
        {
            initialized = false;
        }

        private void Initialize()
        {
            backup = AlgorithmManager.Instance.Table.Clone() as SudokuTable;
            guessOptions = new Queue<GuessOption>();

            for (int cnumber = 2; cnumber <= backup.M; ++cnumber)
            {
                for (int i = 0; i < backup.M; ++i)
                {
                    for (int j = 0; j < backup.M; ++j)
                    {
                        if (backup.Cells[i, j].CandidatesNumber == cnumber)
                        {
                            for (int c = 1; c <= backup.M; ++c)
                            {
                                if (backup.Cells[i, j].IsCandidate(c))
                                {
                                    GuessOption go = new GuessOption();
                                    go.Candidate = c;
                                    go.CandidateCell = new IntPair(i, j);
                                    guessOptions.Enqueue(go);
                                }
                            }
                        }
                    }
                }
            }

            initialized = true;
        }

        public HistoryEntry Guess(bool error)
        {
            if (!initialized) Initialize();

            AlgorithmManager.Instance.Table = backup.Clone() as SudokuTable;
            AlgorithmManager.Instance.SolveDepth = 2;

            if (guessOptions.Count == 0) return null;

            //az előző tipp kizárása a táblából
            if (error && currentTipp != null) backup.Cells[currentTipp.CandidateCell.i, 
                currentTipp.CandidateCell.j].RemoveCandidate(currentTipp.Candidate);

            currentTipp = guessOptions.Dequeue();
            int Candidate = currentTipp.Candidate;
            IntPair CandidatePosition = currentTipp.CandidateCell;

            //a naplóbejegyzés elkészítése
            List<IntPair> rcc;
            List<int> rc;
            GetCandidatesToEliminate(out rc, out rcc, currentTipp.CandidateCell, currentTipp.Candidate);

            HistoryEntry he = new HistoryEntry(new IntPair(currentTipp.CandidateCell.i, currentTipp.CandidateCell.j),
                currentTipp.Candidate, rcc, rc, -1);
            he.AlgorithmNumber = 11;
            he.Depth = 1;
            he.Text = "r" + (CandidatePosition.i + 1).ToString() + "c" + (CandidatePosition.j + 1).ToString() + "=" + Candidate.ToString() + " (Nishio)";

            AlgorithmManager.Instance.Table.setCellData(currentTipp.CandidateCell.i, currentTipp.CandidateCell.j, currentTipp.Candidate);
            AlgorithmManager.Instance.Table.Cells[currentTipp.CandidateCell.i, currentTipp.CandidateCell.j].Foundby = 2;

            for (int k = 0; k < rc.Count; ++k)
            {
                AlgorithmManager.Instance.Table.RemoveCandidate(rcc[k].i, rcc[k].j, rc[k]);
            }

            return he;
        }

        private void GetCandidatesToEliminate(out List<int> rc, out List<IntPair> rcc, IntPair ip, int c)
        {
            rc = new List<int>();
            rcc = new List<IntPair>();

            SudokuTable t = AlgorithmManager.Instance.Table;
            int n = t.N;
            int m = t.M;

            for (int row = 0; row < m; ++row)
            {
                if (t.Cells[row, ip.j].IsCandidate(c) && row != ip.i)
                {
                    rcc.Add(new IntPair(row, ip.j));
                    rc.Add(c);
                }
            }

            for (int col = 0; col < m; ++col)
            {
                if (t.Cells[ip.i, col].IsCandidate(c) && col != ip.j)
                {
                    rcc.Add(new IntPair(ip.i, col));
                    rc.Add(c);
                }
            }

            for (int x = ip.i / n * n; ip.i / n == x / n; ++x)
            {
                for (int y = ip.j / n * n; ip.j / n == y / n; ++y)
                {
                    if (t.Cells[x, y].IsCandidate(c) && ip.i != x && ip.j != y)
                    {
                        rcc.Add(new IntPair(x, y));
                        rc.Add(c);
                    }
                }
            }

            for (int cand = 1; cand < m + 1; ++cand)
            {
                if (cand != c)
                {
                    rcc.Add(new IntPair(ip.i, ip.j));
                    rc.Add(cand);
                }
            }
        }
    }
}
