﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Sudoku
{
    public partial class Hints : Form
    {
        readonly string[] algorithm_names = new string[]{"Naked single", "Hidden single", "Intersection",
            "Naked pair", "Hidden pair", "Naked triple", "Hidden triple", "Naked quad", "Hidden quad", "Hibák", "Crook"};
        SudokuControl sudokucontrol;

        public Hints(SudokuControl sc)
        {
            InitializeComponent();
            sudokucontrol = sc;
        }

        private void Hints_Load(object sender, EventArgs e)
        {
            PopulateHints();
        }

        public void PopulateHints()
        {
            try
            {
                tvHints.Nodes.Clear();
                TreeNode node;

                if (AlgorithmManager.Instance.Findings[9].Count == 0)
                {
                    for (int i = 0; i < AlgorithmManager.Instance.Findings.Count; ++i)
                    {
                        if (AlgorithmManager.Instance.Findings[i].Count > 0)
                        {
                            node = tvHints.Nodes.Add(algorithm_names[i]);

                            for (int k = 0; k < AlgorithmManager.Instance.Findings[i].Count; ++k)
                            {
                                node.Nodes.Add(AlgorithmManager.Instance.Findings[i][k].Text);
                                node.Nodes[k].Tag = 1000 * i + k;
                            }
                        }
                    }
                }
                else
                {
                    node = tvHints.Nodes.Add(algorithm_names[9]);

                    for (int k = 0; k < AlgorithmManager.Instance.Findings[9].Count; ++k)
                    {
                        node.Nodes.Add(AlgorithmManager.Instance.Findings[9][k].Text);
                        node.Nodes[k].Tag = 1000 * 9 + k;
                    }
                }
            }
            catch (ObjectDisposedException) { }
        }

        private void twHints_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (tvHints.SelectedNode.Tag != null)
            {
                int alg_ind = (int)tvHints.SelectedNode.Tag / 1000;
                int pat_ind = (int)tvHints.SelectedNode.Tag % 1000;

                AlgorithmManager.Instance.ChangeShownPattern(alg_ind, pat_ind);
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            if (tvHints.SelectedNode != null && tvHints.SelectedNode.Tag != null)
            {
                int alg_ind = (int)tvHints.SelectedNode.Tag / 1000;
                int pat_ind = (int)tvHints.SelectedNode.Tag % 1000;

                AlgorithmManager.Instance.ApplyPattern(alg_ind, pat_ind);
                PopulateHints();
            }
        }

        private void Hints_Resize(object sender, EventArgs e)
        {
            tvHints.Height = this.Height - 74;
            tvHints.Width = this.Width - 14;
            btnApply.Location = new Point(0, tvHints.Height + 6);
            btnUndo.Location = new Point(this.Width - 131, tvHints.Height + 6);
        }

        private void btnUndo_Click(object sender, EventArgs e)
        {
            AlgorithmManager.Instance.UndoLastPattern();
            PopulateHints();
        }
    }
}
