﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class HiddenErrorPattern : Pattern
    {
        RectangleTypes houseType;
        int houseIndex;

        public HiddenErrorPattern(IntPair cell, int houseIndex, RectangleTypes houseType, int candidate)
        {
            this.algorithmIndex = 9;
            this.houseIndex = houseIndex;
            this.houseType = houseType;
            this.candidateValue = candidate;
            this.greenCandidates = new List<int>();
            this.greenCandidateCells = new List<IntPair>();
            this.greenCandidateCells.Add(cell);
            this.redCandidates = new List<int>();
            this.redCandidateCells = new List<IntPair>();
            this.text = "";

            switch (houseType)
            {
                case RectangleTypes.Row:
                    text = "r" + houseIndex.ToString() + " sorban egy cellában sincs " + SudokuCell.ConvertCellDataToString(candidateValue, AlgorithmManager.Instance.Table.M) + " jelölt";
                    break;
                case RectangleTypes.Column:
                    text = "c" + houseIndex.ToString() + " oszlopban egy cellában sincs " + SudokuCell.ConvertCellDataToString(candidateValue, AlgorithmManager.Instance.Table.M) + " jelölt";
                    break;
                case RectangleTypes.Box:
                    text = "b" + houseIndex.ToString() + " blokkban egy cellában sincs " + SudokuCell.ConvertCellDataToString(candidateValue, AlgorithmManager.Instance.Table.M) + " jelölt";
                    break;
            }
        }

        protected override string getReportEntry(SudokuTable t)
        {
            return "\\highlight1 " + text + "\\highlight0";
        }

        public override void PaintPattern(SudokuTable t)
        {
            for (int i = 0; i < t.M; ++i)
            {
                for (int j = 0; j < t.M; ++j)
                {
                    t.Cells[i, j].BackGroundColor = 3;
                    for (int c = 1; c <= t.M; ++c)
                    {
                        t.Cells[i, j].SetColor(c, 0);
                    }
                }
            }

            switch (houseType)
            {
                case RectangleTypes.Row:
                    for (int row = 0; row < t.M; ++row)
                    {
                        t.Cells[greenCandidateCells[0].i, row].BackGroundColor = 4;
                    }
                    break;
                case RectangleTypes.Column:
                    for (int col = 0; col < t.M; ++col)
                    {

                        t.Cells[col, greenCandidateCells[0].j].BackGroundColor = 4;
                    }
                    break;
                case RectangleTypes.Box:
                    for (int x = greenCandidateCells[0].i / t.N * t.N; greenCandidateCells[0].i / t.N == x / t.N; ++x)
                    {
                        for (int y = greenCandidateCells[0].j / t.N * t.N; greenCandidateCells[0].j / t.N == y / t.N; ++y)
                        {

                            t.Cells[x, y].BackGroundColor = 4;
                        }
                    }
                    break;
            }
        }
    }
}
