﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class HiddenSinglePattern : Pattern
    {
        private RectangleTypes houseType;
        public RectangleTypes HouseType
        {
            get { return houseType; }
            set { houseType = value; }
        }

        /// <summary>
        /// A minta konstruktora
        /// </summary>
        /// <param name="cv">Beirando jelolt erteke (0 ha nincs)</param>
        /// <param name="gc">Zold jeloltek ertekei</param>
        /// <param name="gcc">Zold jeloltek</param>
        /// <param name="rc">Kihuzando jeloltek ertekei</param>
        /// <param name="rcc">Kihuzhato jeloltek mezoi</param>
        public HiddenSinglePattern(int cv, List<int> gc, List<IntPair> gcc, List<int> rc, List<IntPair> rcc, RectangleTypes houseType)
        {
            this.algorithmIndex = 1;
            this.candidateValue = cv;
            this.greenCandidates = gc;
            this.greenCandidateCells = gcc;
            this.redCandidates = rc;
            this.redCandidateCells = rcc;
            this.houseType = houseType;
            this.text = "r" + (greenCandidateCells[0].i + 1).ToString() + "c" + (greenCandidateCells[0].j + 1).ToString() + "=" +
                SudokuCell.ConvertCellDataToString(greenCandidates[0], AlgorithmManager.Instance.Table.M);

            switch (this.HouseType)
            {
                case RectangleTypes.Row:
                    text += " (rejtett a sorban)";
                    break;
                case RectangleTypes.Column:
                    text += " (rejtett az oszlopban)";
                    break;
                default:
                    text += " (rejtett a blokkban)";
                    break;
            }
        }

        protected override string getReportEntry(SudokuTable t)
        {
            string house;
            switch (this.HouseType)
            { 
                case RectangleTypes.Row:
                    house = " a sorban";
                    break;
                case RectangleTypes.Column:
                    house = " az oszlopban";
                    break;
                default:
                    house = " a blokkban";
                    break;
            }

            string message = "r" + (greenCandidateCells[0].i + 1).ToString() +
                "c" + (greenCandidateCells[0].j + 1).ToString() + "=" +
                SudokuCell.ConvertCellDataToString(candidateValue, t.M) + " (rejtett" + house + ")";

            return message;
        }

        private int RowHouse(int i, int j, int N)
        {
            if (i == greenCandidateCells[0].i)
            {
                return 5;
            }
            if (j == greenCandidateCells[0].j || greenCandidateCells[0].i / N == i / N
                && greenCandidateCells[0].j / N == j / N)
            {
                return 4;
            }
            else return 3;
        }

        private int ColumnHouse(int i, int j, int N)
        {
            if (j == greenCandidateCells[0].j)
            {
                return 5;
            }
            if (i == greenCandidateCells[0].i || greenCandidateCells[0].j / N == j / N
                && greenCandidateCells[0].i / N == i / N)
            {
                return 4;
            }
            else return 3;
        }

        private int BlockHouse(int i, int j, int N)
        {
            if (greenCandidateCells[0].i / N == i / N && greenCandidateCells[0].j / N == j / N)
            {
                return 5;
            }
            if (j == greenCandidateCells[0].j || i == greenCandidateCells[0].i)
            {
                return 4;
            }
            else return 3;
        }

        public override void PaintPattern(SudokuTable t)
        {
            for (int i = 0; i < t.M; ++i)
            {
                for (int j = 0; j < t.M; ++j)
                {
                    switch (HouseType)
                    {
                        case RectangleTypes.Box:
                            t.Cells[i, j].BackGroundColor = BlockHouse(i, j, t.N);
                            break;
                        case RectangleTypes.Column:
                            t.Cells[i, j].BackGroundColor = ColumnHouse(i, j, t.N);
                            break;
                        case RectangleTypes.Row:
                            t.Cells[i, j].BackGroundColor = RowHouse(i, j, t.N);
                            break;
                    }

                    for (int c = 1; c <= t.M; ++c)
                    {
                        t.Cells[i, j].SetColor(c, 0);
                    }
                }
            }

            for (int i = 0; i < greenCandidateCells.Count; ++i)
            {
                t.Cells[greenCandidateCells[i].i, greenCandidateCells[i].j].SetColor(greenCandidates[i], 1);
            }

            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                t.Cells[redCandidateCells[i].i, redCandidateCells[i].j].SetColor(redCandidates[i], 2);
            }
        }
    }
}
