﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class IntersectionPattern : Pattern
    {
        private RectangleTypes house;
        public RectangleTypes HouseType
        {
            get { return house; }
            set { house = value; }
        }

        private int tableSize;
        public int TableSize
        {
            get { return tableSize; }
            set { tableSize = value; }
        }

        private int size;

        /// <summary>
        /// A minta konstruktora
        /// </summary>
        /// <param name="cv">Beirando jelolt erteke (0 ha nincs)</param>
        /// <param name="gc">Zold jeloltek ertekei</param>
        /// <param name="gcc">Zold jeloltek</param>
        /// <param name="rc">Kihuzando jeloltek ertekei</param>
        /// <param name="rcc">Kihuzhato jeloltek mezoi</param>
        public IntersectionPattern(int cv, List<int> gc, List<IntPair> gcc, List<int> rc, List<IntPair> rcc, int size)
        {
            this.algorithmIndex = 2;
            this.candidateValue = 0;
            this.greenCandidates = gc;
            this.greenCandidateCells = gcc;
            this.redCandidates = rc;
            this.redCandidateCells = rcc;
            this.text = "";
            this.size = size;

            if (gcc[0].j == gcc[1].j)
            {
                this.text = "-> c" + (greenCandidateCells[0].j + 1).ToString() + "t" + (greenCandidateCells[0].i / size + 1).ToString();
            }
            else
            {
                this.text = "-> r" + (greenCandidateCells[0].i + 1).ToString() + "t" + (greenCandidateCells[0].j / size + 1).ToString();
            }

            text += " (tégla)";
        }

        protected override string getReportEntry(SudokuTable t)
        {
            string text = "";
            if (HouseType == RectangleTypes.Column)
            {
                if (greenCandidateCells[0].j == redCandidateCells[0].j)
                {
                    text = "b" + (greenCandidateCells[0].i / size * size + greenCandidateCells[0].j / size + 1).ToString() + 
                        " blokkban csak a c" + (greenCandidateCells[0].j + 1).ToString() + 
                        "t" + (greenCandidateCells[0].i / size + 1).ToString() + " t\\'e9gl\\'e1ban van a(z) " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " megengedve, ez\\'e9rt az oszlop t\\'f6bbi cell\\'e1j\\'e1b\\'f3l a(z) " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " jel\\'f6lt kih\\'fazhat\\'f3:";
                }
                else
                {
                    text = "c" + (greenCandidateCells[0].j + 1).ToString() + " oszlopban csak a c" + 
                        (greenCandidateCells[0].j + 1).ToString() + "t" + (greenCandidateCells[0].i / size + 1).ToString() +
                        " t\\'e9gl\\'e1ban van a(z) " + SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " megengedve, ez\\'e9rt a blokk t\\'f6bbi cell\\'e1j\\'e1b\\'f3l a(z) " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " jel\\'f6lt kih\\'fazhat\\'f3:";
                }
            }
            else
            {
                if (greenCandidateCells[0].i == redCandidateCells[0].j)
                {
                    text = "b" + (greenCandidateCells[0].i / size * size + greenCandidateCells[0].j / size + 1).ToString() +
                        " blokkban csak a r" + (greenCandidateCells[0].i + 1).ToString() + "t" + (greenCandidateCells[0].j / size + 1).ToString() +
                        " t\\'e9gl\\'e1ban van a(z) " + SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " megengedve, ez\\'e9rt a sor t\\'f6bbi cell\\'e1j\\'e1b\\'f3l a(z) " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " jel\\'f6lt kih\\'fazhat\\'f3:";
                }
                else 
                {
                    text = "r" + (greenCandidateCells[0].i + 1).ToString() + " sorban csak a r" +
                        (greenCandidateCells[0].i + 1).ToString() + "t" + (greenCandidateCells[0].j / size + 1).ToString() +
                        " t\\'e9gl\\'e1ban van a(z) " + SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " megengedve, ez\\'e9rt a blokk t\\'f6bbi cell\\'e1j\\'e1b\\'f3l a(z) " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], t.M) + " jel\\'f6lt kih\\'fazhat\\'f3:";
                }
            }

            text += "\\par" + Environment.NewLine + "\\tabs\\tab ";
            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                text += "r" + (redCandidateCells[i].i + 1).ToString() + "c" + (redCandidateCells[i].j + 1).ToString() + "!=" + SudokuCell.ConvertCellDataToString(redCandidates[i], t.M) +
                    ((i != redCandidateCells.Count - 1) ? "; " : "");
            }
            
            return text;
        }

        public override void PaintPattern(SudokuTable t)
        {
            //elozo minta eltuntetese
            for (int i = 0; i < t.M; ++i)
            {
                for (int j = 0; j < t.M; ++j)
                {
                    t.Cells[i, j].BackGroundColor = 3;
                    for (int c = 1; c <= t.M; ++c)
                    {
                        t.Cells[i, j].SetColor(c, 0);
                    }
                }
            }

            if (HouseType == RectangleTypes.Column)
            {
                if (redCandidateCells[0].j == greenCandidateCells[0].j)
                {
                    for (int i = 0; i < t.M; ++i)
                    {
                        t.Cells[i, greenCandidateCells[0].j].BackGroundColor = 4;
                    }
                    for (int i = 0; i < t.N; ++i)
                    {
                        for (int j = 0; j < t.N; ++j)
                        {
                            t.Cells[greenCandidateCells[0].i / t.N * t.N + i, greenCandidateCells[0].j / t.N * t.N + j].BackGroundColor = 5;
                        }
                    }
                }
                else 
                {
                    for (int i = 0; i < t.N; ++i)
                    {
                        for (int j = 0; j < t.N; ++j)
                        {
                            t.Cells[greenCandidateCells[0].i / t.N * t.N + i, greenCandidateCells[0].j / t.N * t.N + j].BackGroundColor = 4;
                        }
                    }
                    for (int i = 0; i < t.M; ++i)
                    {
                        t.Cells[i, greenCandidateCells[0].j].BackGroundColor = 5;
                    }
                }
            }
            else
            {
                if (redCandidateCells[0].i == greenCandidateCells[0].i)
                {
                    for (int i = 0; i < t.M; ++i)
                    {
                        t.Cells[greenCandidateCells[0].i, i].BackGroundColor = 4;
                    }
                    for (int i = 0; i < t.N; ++i)
                    {
                        for (int j = 0; j < t.N; ++j)
                        {
                            t.Cells[greenCandidateCells[0].i / t.N * t.N + i, greenCandidateCells[0].j / t.N * t.N + j].BackGroundColor = 5;
                        }
                    }
                }
                else
                {
                    for (int i = 0; i < t.N; ++i)
                    {
                        for (int j = 0; j < t.N; ++j)
                        {
                            t.Cells[greenCandidateCells[0].i / t.N * t.N + i, greenCandidateCells[0].j / t.N * t.N + j].BackGroundColor = 4;
                        }
                    }
                    for (int i = 0; i < t.M; ++i)
                    {
                        t.Cells[greenCandidateCells[0].i, i].BackGroundColor = 5;
                    }
                }
            }

            for (int i = 0; i < greenCandidateCells.Count; ++i)
            {
                t.Cells[greenCandidateCells[i].i, greenCandidateCells[i].j].SetColor(greenCandidates[i], 1);
            }

            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                t.Cells[redCandidateCells[i].i, redCandidateCells[i].j].SetColor(redCandidates[i], 2);
            }
        }
    }
}
