﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class NakedPairPattern : Pattern
    {
        private RectangleTypes house;
        public RectangleTypes HouseType
        {
            get { return house; }
            set { house = value; }
        }

        /// <summary>
        /// A minta konstruktora
        /// </summary>
        /// <param name="cv">Beirando jelolt erteke (0 ha nincs)</param>
        /// <param name="gc">Zold jeloltek ertekei</param>
        /// <param name="gcc">Zold jeloltek</param>
        /// <param name="rc">Kihuzando jeloltek ertekei</param>
        /// <param name="rcc">Kihuzhato jeloltek mezoi</param>
        public NakedPairPattern(int cv, List<int> gc, List<IntPair> gcc, List<int> rc, List<IntPair> rcc)
        {
            this.algorithmIndex = 3;
            this.candidateValue = 0;
            this.greenCandidates = gc;
            this.greenCandidateCells = gcc;
            this.redCandidates = rc;
            this.redCandidateCells = rcc;
            this.text = "-> r" + (greenCandidateCells[0].i + 1).ToString() + "c" + (greenCandidateCells[0].j + 1).ToString() +
                ", r" + (greenCandidateCells[1].i + 1).ToString() + "c" + (greenCandidateCells[1].j + 1).ToString() + " (tiszta pár)";
        }

        protected override string getReportEntry(SudokuTable t)
        {
            string text;
            switch (this.HouseType)
            {
                case RectangleTypes.Row:
                    text = "r" + (greenCandidateCells[0].i + 1).ToString() + " sorban r" + (greenCandidateCells[0].i + 1).ToString() + 
                        "c" + (greenCandidateCells[0].j + 1).ToString() + ", r" + (greenCandidateCells[1].i + 1).ToString() + "c" + 
                        (greenCandidateCells[1].j + 1).ToString() + " cell\\'e1k tiszta cellap\\'e1rt alkotnak " + 
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], AlgorithmManager.Instance.Table.M) + ", " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[2], AlgorithmManager.Instance.Table.M) + 
                        " jel\\'f6ltekkel, ez\\'e9rt a sor t\\'f6bbi cell\\'e1j\\'e1b\\'f3l kih\\'fazhat\\'f3k:";
                    break;
                case RectangleTypes.Column:
                    text = "c" + (greenCandidateCells[0].j + 1).ToString() + " oszlopban r" + (greenCandidateCells[0].i + 1).ToString() +
                        "c" + (greenCandidateCells[0].j + 1).ToString() + ", r" + (greenCandidateCells[1].i + 1).ToString() + "c" +
                        (greenCandidateCells[1].j + 1).ToString() + " cell\\'e1k tiszta cellap\\'e1rt alkotnak " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], AlgorithmManager.Instance.Table.M) + ", " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[2], AlgorithmManager.Instance.Table.M) + 
                        " jel\\'f6ltekkel, ez\\'e9rt az oszlop t\\'f6bbi cell\\'e1j\\'e1b\\'f3l kih\\'fazhat\\'f3k:";
                    break;
                default:
                    int blokk = greenCandidateCells[0].i / t.N * t.N + greenCandidateCells[0].j / t.N + 1;
                    text = "b" + blokk.ToString() + " blokkban r" + (greenCandidateCells[0].i + 1).ToString() +
                        "c" + (greenCandidateCells[0].j + 1).ToString() + ", r" + (greenCandidateCells[1].i + 1).ToString() + "c" +
                        (greenCandidateCells[1].j + 1).ToString() + " cell\\'e1k tiszta cellap\\'e1rt alkotnak " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[0], AlgorithmManager.Instance.Table.M) + ", " +
                        SudokuCell.ConvertCellDataToString(greenCandidates[2], AlgorithmManager.Instance.Table.M) + 
                        " jel\\'f6ltekkel, ez\\'e9rt a blokk t\\'f6bbi cell\\'e1j\\'e1b\\'f3l kih\\'fazhat\\'f3k:";
                    break;
            }

            text += "\\par" + Environment.NewLine + "\\tabs\\tab ";
            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                text += "r" + (redCandidateCells[i].i + 1).ToString() + "c" + (redCandidateCells[i].j + 1).ToString() + "!=" + SudokuCell.ConvertCellDataToString(redCandidates[i], t.M) +
                    ((i != redCandidateCells.Count - 1) ? "; " : "");
            }

            return text;
        }

        public override void PaintPattern(SudokuTable t)
        {
            for (int i = 0; i < t.M; ++i)
            {
                for (int j = 0; j < t.M; ++j)
                {
                    switch (HouseType)
                    {
                        case RectangleTypes.Box:
                            if (i / t.N == greenCandidateCells[0].i / t.N && j / t.N == greenCandidateCells[0].j / t.N)
                            {
                                if (i == greenCandidateCells[0].i && j == greenCandidateCells[0].j
                                    || i == greenCandidateCells[1].i && j == greenCandidateCells[1].j)
                                {
                                    t.Cells[i, j].BackGroundColor = 5;
                                }
                                else t.Cells[i, j].BackGroundColor = 4;
                            }
                            else t.Cells[i, j].BackGroundColor = 3;
                            break;
                        case RectangleTypes.Column:
                            if (j == greenCandidateCells[0].j)
                            {
                                if (i == greenCandidateCells[0].i && j == greenCandidateCells[0].j
                                    || i == greenCandidateCells[1].i && j == greenCandidateCells[1].j)
                                {
                                    t.Cells[i, j].BackGroundColor = 5;
                                }
                                else t.Cells[i, j].BackGroundColor = 4;
                            }
                            else t.Cells[i, j].BackGroundColor = 3;
                            break;
                        case RectangleTypes.Row:
                            if (i == greenCandidateCells[0].i)
                            {
                                if (i == greenCandidateCells[0].i && j == greenCandidateCells[0].j
                                    || i == greenCandidateCells[1].i && j == greenCandidateCells[1].j)
                                {
                                    t.Cells[i, j].BackGroundColor = 5;
                                }
                                else t.Cells[i, j].BackGroundColor = 4;
                            }
                            else t.Cells[i, j].BackGroundColor = 3;
                            break;
                    }

                    for (int c = 1; c <= t.M; ++c)
                    {
                        t.Cells[i, j].SetColor(c, 0);
                    }
                }
            }

            for (int i = 0; i < greenCandidateCells.Count; ++i)
            {
                t.Cells[greenCandidateCells[i].i, greenCandidateCells[i].j].SetColor(greenCandidates[i], 1);
            }

            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                t.Cells[redCandidateCells[i].i, redCandidateCells[i].j].SetColor(redCandidates[i], 2);
            }
        }
    }
}
