﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Sudoku
{
    public abstract class Pattern
    {
        protected List<IntPair> greenCandidateCells;
        protected List<int> greenCandidates;
        protected List<IntPair> redCandidateCells;
        protected List<int> redCandidates;
        protected string text;

        protected int algorithmIndex;
        public int AlgorithmIndex
        {
            get { return algorithmIndex; }
        }

        protected int candidateValue;
        public int CandidateValue
        {
            get { return candidateValue; }
        }

        public IntPair CandidatePosition
        {
            get { return greenCandidateCells[0]; }
        }

        public string Text
        {
            get { return text; }
            set { text = value; }
        } 

        public bool ApplyPattern(SudokuTable t, out HistoryEntry historyEntry)
        {
            historyEntry = new HistoryEntry(greenCandidateCells, greenCandidates, redCandidateCells, redCandidates);
            historyEntry.Text = getReportEntry(t);
            historyEntry.AlgorithmNumber = algorithmIndex;

            IntPair ip;
            int n, gn;
            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                ip = redCandidateCells[i];
                n = redCandidates[i];
                gn = -1;
                if (candidateValue > 0) gn = candidateValue;

                //Ha mar egy korabbi lepes soran kihuztuk a jeloltet ugorjuk at
                if (t.Cells[ip.i, ip.j].IsCandidate(n) && !t.Cells[ip.i, ip.j].IsData())
                {
                    if (!t.RemoveCandidate(ip.i, ip.j, n))
                    {
                        historyEntry.Text += "\\par\\tabs\\tab\\highlight1 -> Hiba: r" + (redCandidateCells[i].i + 1).ToString() + "c" + (redCandidateCells[i].j + 1).ToString() +
                            " <-- " + SudokuCell.ConvertCellDataToString(n, t.M) + " kihúzásakor\\highlight0";
                        historyEntry.AlgorithmNumber = 9;
                    }
                    else
                    {
                        if (AlgorithmManager.Instance.SolveState == SolveStates.RozsaSolve || AlgorithmManager.Instance.SolveState == SolveStates.UserSolve)
                        {
                            IntPair greenCell;
                            if (greenCandidateCells.Count > 1) greenCell = new IntPair(-1, -1);
                            else greenCell = greenCandidateCells[0];
                            historyEntry.Text += AlgorithmManager.Instance.TestCell(redCandidateCells[i], greenCell, n, gn);
                        }
                    }
                }
            }

            foreach (IntPair cell in greenCandidateCells)
            {
                for (int i = 1; i <= t.M; ++i)
                {
                    if (t.Cells[cell.i, cell.j].IsCandidate(i))
                    {
                        t.Cells[cell.i, cell.j].SetColor(i, 0);
                    }
                }
            }

            if (candidateValue != 0)
            {
                t.setCellData(greenCandidateCells[0].i, greenCandidateCells[0].j, candidateValue);
                t.Cells[greenCandidateCells[0].i, greenCandidateCells[0].j].Foundby = AlgorithmManager.Instance.SolveDepth;
            }

            foreach (SudokuCell cell in t.Cells)
            {
                cell.BackGroundColor = 3;
            }

            if (historyEntry.AlgorithmNumber == 9)
                return false;
            else
                return true;
        }

        protected abstract string getReportEntry(SudokuTable t);

        public abstract void PaintPattern(SudokuTable t);

        public bool Equals(Pattern p)
        {
            if (p.greenCandidateCells == null || p.greenCandidates == null || p.redCandidateCells == null || p.redCandidates == null ||
                greenCandidates == null || greenCandidateCells == null || redCandidateCells == null || redCandidates == null)
            {
                return false;
            }

            if (this.redCandidateCells.Count != p.redCandidateCells.Count ||
                this.greenCandidateCells.Count != p.greenCandidateCells.Count)
            {
                return false;
            }

            for (int i = 0; i < greenCandidateCells.Count; ++i)
            {
                if (this.greenCandidateCells[i].i != p.greenCandidateCells[i].i && this.greenCandidateCells[i].j != p.greenCandidateCells[i].j || 
                    this.greenCandidates[i] != p.greenCandidates[i])
                {
                    return false;
                }
            }

            for (int i = 0; i < redCandidateCells.Count; ++i)
            {
                if (this.redCandidateCells[i].i != p.redCandidateCells[i].i && this.redCandidateCells[i].j != p.redCandidateCells[i].j || 
                    this.redCandidates[i] != p.redCandidates[i])
                {
                    return false;
                }
            }

            return true;
        }

        public bool IsValid() 
        {
            for (int i = 0; i < redCandidateCells.Count; ++i)
            { 
                if (AlgorithmManager.Instance.Table.Cells[redCandidateCells[i].i, redCandidateCells[i].j].IsCandidate(redCandidates[i]))
                    return true;
            }
            return false;
        }
    }
}
