﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class CandidatesTable
    {
        List<int>[,] data;
        bool[,] isData;
        int[,] foundBy;
        int dataInTable;
        int m;

        public CandidatesTable(SudokuTable t)
        {
            m = t.M;
            dataInTable = 0;
            data = new List<int>[t.M, t.M];
            isData = new bool[t.M, t.M];
            foundBy = new int[t.M, t.M];

            for (int i = 0; i < t.M; ++i)
            {
                for (int j = 0; j < t.M; ++j)
                {
                    data[i, j] = new List<int>();
                    if (t.Cells[i, j].IsData())
                    {
                        data[i, j].Add(t.Cells[i, j].Data);
                        isData[i, j] = true;
                        foundBy[i, j] = t.Cells[i, j].Foundby;
                        ++dataInTable;
                    }
                    else
                    {
                        for (int c = 1; c <= t.M; ++c)
                        {
                            if (t.Cells[i, j].IsCandidate(c))
                            {
                                data[i, j].Add(c);
                            }
                        }
                        isData[i, j] = false;
                    }
                }
            }
        }

        public string getCandidateTable()
        {
            string tableString = Environment.NewLine + "\\par\\par\\f2";

            tableString += "    | ";
            for (int t = 1; t <= m; ++t)
            {
                tableString += "c" + t.ToString();
                for (int k = 1; k < m - t / 10; ++k) tableString += " ";
            }
            tableString += Environment.NewLine + "\\par ---|-";
            for (int t = 0; t < m * m + m; ++t) tableString += "-";
            for (int t = 1; t <= m; ++t) 
            {
                tableString += Environment.NewLine + "\\par r" + t.ToString() + ((t / 10 == 0) ? " " : "") + "| ";
                for (int k = 0; k < m; ++k)
                {
                    for (int l = 0; l < data[t - 1, k].Count; ++l) tableString += SudokuCell.ConvertCellDataToString(data[t - 1, k][l], m);
                    for (int l = data[t - 1, k].Count; l < m + 1; ++l) tableString += " ";
                }
            }
            tableString += "\\f0";

            return tableString;
        }

        public string getSimpleTable()
        {
            string tableString = Environment.NewLine + "\\par\\f2 ";

            for (int t = 1; t <= m; ++t)
            {
                tableString += "\\par ";
                for (int k = 0; k < m; ++k)
                {
                    tableString += SudokuCell.ConvertCellDataToString(data[t - 1, k][0], m);
                }
            }
            tableString += "\\f0";

            return tableString;
        }
    }
}
