﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    public class HistoryEntry
    {
        #region properties

        private List<IntPair> greenCandidateCells;
        private List<int> greenCandidates;
        private List<IntPair> redCandidateCells;
        private List<int> redCandidates;

        private CandidatesTable candidateTable;
        public CandidatesTable CandidatesTable
        {
            get { return candidateTable; }
            set { candidateTable = value; }
        }

        private int step;
        public int Step
        {
            get { return step; }
            set { step = value; }
        }

        private string text;
        public string Text
        {
            get { return text; }
            set { text = value; }
        }

        private int depth;
        public int Depth
        {
            get { return depth; }
            set { depth = value; }
        }

        private int algorithmNumber;
        public int AlgorithmNumber
        {
            get { return algorithmNumber; }
            set { algorithmNumber = value; }
        }

        #endregion

        public HistoryEntry(List<IntPair> gcc, List<int> gc, List<IntPair> rcc, List<int> rc)
        {
            greenCandidateCells = gcc;
            greenCandidates = gc;
            redCandidateCells = rcc;
            redCandidates = rc;
            depth = 0;
            text = "";
        }

        public HistoryEntry(IntPair rcc, int rc, int algNumber)
        {
            redCandidateCells = new List<IntPair>();
            redCandidateCells.Add(rcc);
            redCandidates = new List<int>();
            redCandidates.Add(rc);
            greenCandidateCells = new List<IntPair>();
            greenCandidates = new List<int>();
            algorithmNumber = algNumber;
            depth = 0;
            text = "";
        }

        public HistoryEntry(IntPair gcc, int gc, List<IntPair> rcc, List<int> rc, int algNumber)
        {
            redCandidates = rc;
            redCandidateCells = rcc;
            greenCandidateCells = new List<IntPair>();
            greenCandidateCells.Add(gcc);
            greenCandidates = new List<int>();
            greenCandidates.Add(gc);
            algorithmNumber = algNumber;
            depth = 0;
            text = "";
        }

        public void Undo()
        {
            for (int k = 0; k < greenCandidates.Count; ++k)
            {
                AlgorithmManager.Instance.Table.removeCellData(greenCandidateCells[k].i, greenCandidateCells[k].j, greenCandidates[k]);
                AlgorithmManager.Instance.Table.Cells[greenCandidateCells[k].i, greenCandidateCells[k].j].AddCandidate(greenCandidates[k]);
                AlgorithmManager.Instance.Table.Cells[greenCandidateCells[k].i, greenCandidateCells[k].j].SetColor(greenCandidates[k], 0);
            }

            for (int k = 0; k < redCandidates.Count; ++k)
            {
                AlgorithmManager.Instance.Table.Cells[redCandidateCells[k].i, redCandidateCells[k].j].AddCandidate(redCandidates[k]);
                AlgorithmManager.Instance.Table.Cells[redCandidateCells[k].i, redCandidateCells[k].j].SetColor(redCandidates[k], 0);
            }
        }

        public void GuessType(int LastAlgorithmIndex, int EntryCount)
        {
            int c = greenCandidates[0];
            int i = greenCandidateCells[0].i;
            int j = greenCandidateCells[0].j;

            bool A0 = true, A1 = false, A2 = false;
            string entryText = "";
            foreach (Pattern p in AlgorithmManager.Instance.Findings[0])
            {
                if (p.CandidateValue == c && p.CandidatePosition.i == i && p.CandidatePosition.j == j)
                {
                    A0 = false;
                    A1 = true;
                    entryText = p.Text;
                }
            }
            if (A0)
            {
                foreach (Pattern p in AlgorithmManager.Instance.Findings[1])
                {
                    if (p.CandidateValue == c && p.CandidatePosition.i == i && p.CandidatePosition.j == j)
                    {
                        A0 = false;
                        A2 = true;
                        entryText = p.Text;
                    }
                }
            }

            if (A0 && (EntryCount == 0 || LastAlgorithmIndex == -1) && AlgorithmManager.Instance.SolveState == SolveStates.WritingData)
            {
                AlgorithmNumber = -1;
                Text = "r" + (i + 1).ToString() + "c" + (j + 1).ToString() + "=" + SudokuCell.ConvertCellDataToString(c, AlgorithmManager.Instance.Table.M) + " (adat)" + Text;
                Depth = 0;
                AlgorithmManager.Instance.Table.Cells[i, j].Foundby = 0;
                AlgorithmManager.Instance.SolveDepth = 1;
            }
            else if (A0)
            {
                AlgorithmNumber = 10;
                Text = "r" + (i + 1).ToString() + "c" + (j + 1).ToString() + "=" + SudokuCell.ConvertCellDataToString(c, AlgorithmManager.Instance.Table.M) + " (Crook)" + Text;
                Depth = AlgorithmManager.Instance.SolveDepth;
                AlgorithmManager.Instance.Table.Cells[i, j].Foundby = AlgorithmManager.Instance.SolveDepth;
            }
            if (A1)
            {
                AlgorithmNumber = 0;
                Text = entryText + Text;
                Depth = AlgorithmManager.Instance.SolveDepth;
                AlgorithmManager.Instance.Table.Cells[i, j].Foundby = AlgorithmManager.Instance.SolveDepth;
            }
            if (A2)
            {
                AlgorithmNumber = 1;
                Text = entryText + Text;
                Depth = AlgorithmManager.Instance.SolveDepth;
                AlgorithmManager.Instance.Table.Cells[i, j].Foundby = AlgorithmManager.Instance.SolveDepth;
            }
        }
    }
}
