﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Sudoku
{
    class SudokuHistory
    {
        string[] algorithmNames = { "Tiszta", "Rejtett", "T\\'e9gla", "Tiszta p\\'e1r", "Rejtett p\\'e1r", "Tiszta h\\'e1rmas",
                                          "Rejtett h\\'e1rmas", "Tiszta n\\'e9gyes", "Rejtett n\\'e9gyes", "Hiba", "Crook", "Nishio"};

        private int tableSize;

        private List<HistoryEntry> history;

        private bool showCandidatesTable;
        public bool ShowCandidatesTable
        {
            get { return showCandidatesTable; }
            set { showCandidatesTable = value; }
        }

        private bool showSimpleResult;
        public bool ShowSimpleResult
        {
            get { return showSimpleResult; }
            set { showSimpleResult = value; }
        }

        public bool IsEmpty
        {
            get { return history.Count == 0 || history[history.Count - 1].AlgorithmNumber == -1; }
        }

        public bool UndoIsAllowed
        {
            get 
            {
                if (history.Count == 0) return false;
                for (int i = history.Count - 1; i >= 0; --i)
                { 
                    if (history[i].AlgorithmNumber != 10 && history[i].AlgorithmNumber > 0)
                        return false;
                }
                return true;
            }
        }

        public HistoryEntry LastEntry
        {
            get { return history[history.Count - 1]; }
        }

        public SudokuHistory()
        {
            showCandidatesTable = true;
            history = new List<HistoryEntry>();
        }

        public void ClearHistory()
        {
            tableSize = AlgorithmManager.Instance.Table.M;
            history = new List<HistoryEntry>();
        }

        public void ResetHistory()
        {
            for (int i = history.Count - 1; i >= 0 && history[i].AlgorithmNumber != -1; --i)
            {
                history.RemoveAt(i);
            }
        }

        public void AddEntry(HistoryEntry newEntry, bool tryGuess)
        {
            if (tryGuess)
            {
                if (!IsEmpty)
                {
                    newEntry.GuessType(LastEntry.AlgorithmNumber, history.Count);
                }
                else
                {
                    newEntry.GuessType(-1, 0);
                }

                if (newEntry.AlgorithmNumber != 10 && (history.Count == 0 || LastEntry.AlgorithmNumber == newEntry.AlgorithmNumber))
                {
                    newEntry.CandidatesTable = null;
                }
            }

            if (!tryGuess && newEntry.AlgorithmNumber < 0)
            {
                newEntry.Step = history.Count + 1;
            }
            else if (newEntry.AlgorithmNumber == 9)
            {
                newEntry.Step = AlgorithmManager.Instance.Table.DataInTable + 1;            
            }
            else
            {
                newEntry.Step = AlgorithmManager.Instance.Table.DataInTable;
            }

            history.Add(newEntry);
        }

        public void RemoveLastEntry()
        {
            //if (history[history.Count - 1].AlgorithmNumber != -1)
            {
                history[history.Count - 1].Undo();
                history.RemoveAt(history.Count - 1);
                if (history.Count == 0) AlgorithmManager.Instance.SolveState = SolveStates.WritingData;
            }
        }

        public string GetReport(int tableSize)
        {
            string report = "{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang1033{\\fonttbl{\\f0\\fnil\\fcharset0 Times New Roman;}{\\f1\\fnil\\fcharset0 Calibri;}{\\f2\\fnil\\fcharset0 Courier;}}" 
            + Environment.NewLine + "{\\colortbl ;\\red255\\green0\\blue0;}"
            + Environment.NewLine + "{\\*\\generator Msftedit 5.41.21.2509;}\\viewkind4\\uc1\\pard\\sl276\\slmult1\\lang9\\b\\f0\\fs24" + Environment.NewLine;
            int step = 1, substep = 0;
            int lastAlgorithm = -2;
            int sudokuDifficulty = 0, startDataCount = 0;
            string tabs = "";

            report += " A0 algoritmus (adat):\\b0";
            for (int i = 0; i < history.Count; ++i)
            {
                if (history[i].AlgorithmNumber == -2) continue;

                if (history[i].Depth > 0)
                {
                    tabs = "";
                    for (int ind = 0; ind < history[i].Depth - 1; ++ind)
                    {
                        tabs += "  ";
                    }
                }

                if (history[i].CandidatesTable != null && showCandidatesTable && history[i].AlgorithmNumber != 9)
                    report += history[i].CandidatesTable.getCandidateTable() + Environment.NewLine;
                
                if (history[i].AlgorithmNumber != lastAlgorithm && history[i].AlgorithmNumber >= 0 && history[i].AlgorithmNumber != 9)
                {
                    try
                    {
                        report += "\\par\\par" + tabs + "\\b" + Environment.NewLine + "A" + (history[i].AlgorithmNumber > 8 ? 10 : (history[i].AlgorithmNumber + 1)).ToString() + " algoritmus (" + 
                            algorithmNames[history[i].AlgorithmNumber] + "):\\b0" + Environment.NewLine;
                        lastAlgorithm = history[i].AlgorithmNumber;
                    }
                    catch (IndexOutOfRangeException)
                    {
                        report += tabs + "Unknown Algorithm\\par" + Environment.NewLine;
                    }
                }

                if (history[i].AlgorithmNumber < 0) ++startDataCount;

                if (history[i].AlgorithmNumber < 2)
                {
                    substep = 0;
                    report += "\\par" + tabs + Environment.NewLine + "  " + history[i].Step.ToString() + ". l\\'e9p\\'e9s: " + 
                        history[i].Text.Replace("\\tabs", tabs);
                    ++step;                
                }
                else
                {
                    ++substep;
                    report += "\\par" + tabs + Environment.NewLine + "  " + (history[i].Step + 1).ToString() + "-" + substep.ToString() +
                        ". all\\'e9p\\'e9s: " + history[i].Text.Replace("\\tabs", tabs);
                }

                if (history[i].AlgorithmNumber > sudokuDifficulty) sudokuDifficulty = history[i].AlgorithmNumber;
            }

            if (AlgorithmManager.Instance.Table.DataInTable == tableSize * tableSize && LastEntry.Step != tableSize * tableSize)
            {
                report += "\\par" + Environment.NewLine + "\\par\\b Dancing links...\\b0 ";
            }

            if (AlgorithmManager.Instance.Table.DataInTable == tableSize * tableSize)
            {
                CandidatesTable result = new CandidatesTable(AlgorithmManager.Instance.Table);
                report += Environment.NewLine + "\\par" + Environment.NewLine + "\\par\\b A megold\\'e1s: \\b0" + 
                    (showSimpleResult ? result.getSimpleTable() : result.getCandidateTable());


                report += Environment.NewLine + "\\par" + Environment.NewLine + "\\par\\b A rejtv\\'e9ny t\\'edpusa: \\b0 " + AlgorithmManager.Instance.Table.N +
                    "A" + (sudokuDifficulty < 10 ? (sudokuDifficulty + 1).ToString() : "10") + "-" + startDataCount.ToString();
            } 
            else if (!IsEmpty && LastEntry.AlgorithmNumber == 9)
            {
                report += Environment.NewLine + "\\par" + Environment.NewLine + "\\par\\b A rejtv\\'e9ny t\\'edpusa:\\b0 " + AlgorithmManager.Instance.Table.N.ToString() + 
                    "U0-" + startDataCount.ToString();
            }

            report += Environment.NewLine + "}" + Environment.NewLine + "NIL";

            return report;
        }
    }
}
