using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Sudoku
{
    public class SudokuCell : ICloneable
    {
        public delegate void CellChangedEventType(object o);
        public event CellChangedEventType CellChanged;

        #region Variables: candidatesnumber, candidate, eliminated Properties: M, Data, CandidatesNumber,

        int backGroundColor;

        public int BackGroundColor
        {
            get { return backGroundColor; }
            set { backGroundColor = value; }
        }

        int candidatesnumber;

        public int CandidatesNumber
        {
            get { return candidatesnumber; }
        }

        int m = 9;

        public int M
        {
            get { return m; }
        }

        int[] eliminated;

        bool[] candidate;

        int[] colors;

        public int GetColor(int candidate)
        {
            return colors[candidate - 1];
        }

        public void SetColor(int candidate, int color)
        {
            colors[candidate - 1] = color;
        }

        int data = 0;
        public int Data
        {
            get { return data; }
            internal set { data = value; }
        }

        int foundby = 0;
        public int Foundby
        {
            get { return foundby; }
            set { foundby = value; }
        }

        #endregion

        protected void OnCellChanged(object o)
        {
            if (CellChanged != null)
            {
                CellChanged(o);
            }
        }

        public SudokuCell()
        {
            SudokuCellInit();
        }

        public SudokuCell(int size)
        {
            m = size;
            SudokuCellInit();
        }

        private void SudokuCellInit()
        {
            candidate = new bool[m];
            eliminated = new int[m];
            colors = new int[m];

            BackGroundColor = 3;
            for (int i = 0; i < m; i++)
            {
                candidate[i] = true;
                eliminated[i] = 0;
                colors[i] = 0;
            //    candidate_list.Add(i + 1); //itt a tnyleges jelltek vannak
            }
            candidatesnumber = m;
            data = 0;
        }

        public void AddCandidate(int c)
        {
            c--; //eltols a tmbindexek miatt
            eliminated[c]--;
            if (!candidate[c] && (eliminated[c] == 0))
            {
                candidatesnumber++;
                candidate[c] = true;
            }
        }

        public void RemoveCandidate(int c)
        {
            c--;
            eliminated[c]++;
            if (candidate[c])
            {
                candidatesnumber--;
                candidate[c] = false;
            }
        }

        public bool IsCandidate(int c)
        {
            c--; //eltols a tmbindexek miatt
            if (c >= 0 && c < m)
            {
                return candidate[c];
            }
            return false;
        }

        public void ResetCell()
        {
            SudokuCellInit();
            GC.Collect();
        }

        public bool IsData()
        {
            return (data != 0);
        }

        #region statikus fggvnyek

        public static bool IsLegalCharacter(char ch, int m)
        {
            ch = Char.ToUpper(ch);
            return ((m <= 9) ? ((int)ch > 48 && (int)ch < m + 49) : ((int)ch > 64 && (int)ch < m + 65));
        }

        public static bool CompareByCandidates(SudokuCell cell1, SudokuCell cell2)
        {
            if (cell1.candidatesnumber != cell2.candidatesnumber || cell1.m != cell2.m) return false;
            for (int i = 0; i < cell1.m; i++)
            {
                if (cell1.candidate[i] != cell2.candidate[i]) return false;
            }
            return true;
        }
        /// <summary>
        /// Rszhalmaza-e cell2 cell1-nek jelltek szerint.
        /// </summary>
        /// <param name="cell1">Halmaz</param>
        /// <param name="cell2">Rszhalmaz</param>
        /// <returns>Igaz/Hamis</returns>
        public static bool Subset(SudokuCell cell1, SudokuCell cell2)
        {
            if (cell1.candidatesnumber < cell2.candidatesnumber) return false;
            for (int i = 0; i < cell1.m; i++)
            {
                if (cell2.candidate[i] && !cell1.candidate[i]) return false;
            }
            return true;
        }

        public static bool Disjunct(SudokuCell cell1, SudokuCell cell2)
        {
            bool l = true;
            for (int c = 1; c <= ((cell1.m < cell2.m) ? cell1.m : cell2.m); c++)
            {
                l = l && ((cell1.IsCandidate(c) ^ cell2.IsCandidate(c)) || (cell1.IsCandidate(c) == false && cell2.IsCandidate(c) == false));
            }
            return l;
        }

        public static string ConvertCellDataToString(int c, int m)
        {
            return ((m <= 9) ? c.ToString() : ((char)(c + 64)).ToString());
        }

        public static int ConvertCharToCellData(char ch)
        {
            ch = Char.ToUpper(ch);
            return (((int)ch > 47 && (int)ch < 58) ? ((int)ch) - 48 : ((int)ch) - 64);
        }

        public static bool[] union(SudokuCell cell1, SudokuCell cell2, out int count)
        {
            bool[] u = new bool[cell1.m + 1];
            count = 0;
            for (int i = 1; i <= cell1.m; ++i)
            {
                if (cell1.IsCandidate(i) || cell2.IsCandidate(i))
                {
                    u[i] = true;
                    ++count;
                }
            }
            return u;
        }

        public static bool[] union(SudokuCell cell1, bool[] cell2, out int count)
        {
            bool[] u = new bool[cell1.m + 1];
            count = 0;
            for (int i = 1; i <= cell1.m; ++i)
            {
                if (cell1.IsCandidate(i) || cell2[i])
                {
                    u[i] = true;
                    ++count;
                }
            }
            return u;
        }

        #endregion

        #region ICloneable Members

        public object Clone()
        {
            SudokuCell sc = new SudokuCell(m);
            sc.data = data;
            sc.foundby = foundby;
            sc.candidatesnumber = candidatesnumber;
            sc.BackGroundColor = BackGroundColor;
            for (int i = 0; i < m; i++)
            {
                sc.candidate[i] = candidate[i];
                sc.eliminated[i] = eliminated[i];
                sc.colors[i] = colors[i];
            }
            //sc.candidate_list = candidate_list;
            return sc;
        }

        #endregion
    }
}
