using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;

namespace Sudoku
{
    public enum RectangleTypes { Row, Column, Box }

    public class SudokuTable : ICloneable
    {
        #region vltozk: n, m, properties[propnum], filename, cells[m, m]; tulajdonsgok: N, M, Cells, Properties, Filename

        int data_in_table = 0;

        public int DataInTable
        {
            get { return data_in_table; }
            set { data_in_table = value; }
        }

        const int propnum = 8;
        
        string[] properties;

        public string[] Properties
        {
            get { return properties; }
            set { properties = value; }
        }

        string filename;

        public string Filename
        {
            get { return filename; }
        }

        SudokuCell[,] cells;

        public SudokuCell[,] Cells
        {
            get { return cells; }
            set { cells = value; }
        }

        int n = 3;

        public int N
        {
            get { return n; }
            set { n = value; m = n * n; }
        }

        int m = 9;

        public int M
        {
            get { return m; }
            private set { m = value; }
        }

        #endregion

        #region konstruktorok, init

        public SudokuTable()
        {
            SudokuTableInit();
        }

        public SudokuTable(int nsize)
        {
            N = nsize;
            SudokuTableInit();
        }

        private void SudokuTableInit()
        {
            properties = new string[propnum];
            cells = new SudokuCell[m, m];
            for (int i = 0; i < m; i++)
                for (int j = 0; j < m; j++)
                    cells[i, j] = new SudokuCell(m);
        }

        #endregion

        #region cellk adatainak belltsa

        public void setCellData(int i, int j, int c)
        {
            
            if (!cells[i,j].IsData())
            {
                if (c != 0)
                {
                    data_in_table++;
                }
            }
            else
            {
                if (c == 0)
                {
                    data_in_table--;
                }
            }
            cells[i, j].Data = c;

            foreach (SudokuCell cell in cells)
            {
                cell.BackGroundColor = 3;
            }
        }

        public void removeCellData(int i, int j, int c)
        {
            if (cells[i, j].IsData())
            {
                if (c != 0)
                {
                    data_in_table--;
                }
            }
            cells[i, j].Data = 0;
        }

        public int getCellData(int i, int j)
        {
            return cells[i, j].Data;
        }

        //mutatja, hogy leglis-e a trls, nem lett-e hibs a tbla (true - OK, false - hibs)
        //de mindenkppen trl
        public bool RemoveCandidate(int i, int j, int c)
        {
            cells[i, j].RemoveCandidate(c);
            return (cells[i, j].CandidatesNumber != 0);
        }

        public void HideShownPattern()
        {
            for (int i = 0; i < M; ++i)
            {
                for (int j = 0; j < M; ++j)
                {
                    Cells[i, j].BackGroundColor = 3;
                    for (int c = 1; c <= M; ++c)
                    {
                        Cells[i, j].SetColor(c, 0);
                    }
                }
            }
        }

        #endregion

        #region jelltek kizrsa, hozzadsa

        //false, ha lett 0 jellt egy cellban
        public bool DeleteCandidateFromRow(int x, int y, int candidate)
        {
            bool l = true;
            for (int j = 0; j < y; j++)
            {
                l = l & RemoveCandidate(x, j, candidate);// cells[x, j].RemoveCandidate(candidate);
            }
            for (int j = y + 1; j < m; j++)
            {
                l = l & RemoveCandidate(x, j, candidate);// cells[x, j].RemoveCandidate(candidate);
            }
            return l;
        }

        public bool DeleteCandidateFromColumn(int x, int y, int candidate)
        {
            bool l = true;
            for (int i = 0; i < x; i++)
            {
                l = l & RemoveCandidate(i, y, candidate);// cells[i, y].RemoveCandidate(candidate);
            }
            for (int i = x + 1; i < m; i++)
            {
                l = l & RemoveCandidate(i, y, candidate); //cells[i, y].RemoveCandidate(candidate);
            }
            return l;
        }

        public bool DeleteCandidateFromBlock(int x, int y, int candidate)
        {
            bool l = true;
            int bx = (x / n) * n;
            int by = (y / n) * n;
            for (int i = bx; i < bx + n; i++)
            {
                for (int j = by; j < by + n; j++)
                {
                    if (i != x && j != y)
                    {
                        l = l & RemoveCandidate(i, j, candidate);// cells[i, j].RemoveCandidate(candidate);
                    }
                }
            }
            return l;
        }

        public bool DeleteCandidateFromThreeHouse(int x, int y, int candidate)
        {
            bool l = true;
            l = l & DeleteCandidateFromRow(x, y, candidate);
            l = l & DeleteCandidateFromColumn(x, y, candidate);
            l = l & DeleteCandidateFromBlock(x, y, candidate);
            return l;
        }

        public void AddCandidateToRow(int x, int y, int candidate)
        {
            for (int j = 0; j < y; j++)
            {
                cells[x, j].AddCandidate(candidate);
            }
            for (int j = y + 1; j < m; j++)
            {
                cells[x, j].AddCandidate(candidate);
            }
        }

        public void AddCandidateToColumn(int x, int y, int candidate)
        {
            for (int i = 0; i < x; i++)
            {
                cells[i, y].AddCandidate(candidate);
            }
            for (int i = x + 1; i < m; i++)
            {
                cells[i, y].AddCandidate(candidate);
            }
        }

        public void AddCandidateToBlock(int x, int y, int candidate)
        {
            int bx = (x / n) * n;
            int by = (y / n) * n;
            for (int i = bx; i < bx + n; i++)
            {
                for (int j = by; j < by + n; j++)
                {
                    if (i != x && j != y)
                    {
                        cells[i, j].AddCandidate(candidate);
                    }
                }
            }
        }

        public void AddCandidateToThreeHouse(int x, int y, int candidate)
        {
            AddCandidateToRow(x, y, candidate);
            AddCandidateToColumn(x, y, candidate);
            AddCandidateToBlock(x, y, candidate);
        }

        #endregion

        #region stringkezels: betlts string-bl, ments string-be, jellttbla ellltsa string-knt

        public bool ValidString(string str)
        {
            str = str.Trim();
            return (str.Length == 81 && System.Text.RegularExpressions.Regex.IsMatch(str, @"[.0-9]{81}")) || (str.Length == 89 && System.Text.RegularExpressions.Regex.IsMatch(str, @"[_+0-9]{89}"));
        }

        public bool LoadFromString(string str)
        {
            if (n != 3)
            {
                return false;
            }
            str = str.Trim();
            if (!(str.Length == 81 || str.Length == 89))
            {
                return false;
            }
            try
            {
                if (str.Length == 89)
                {
                    str = str.Replace('_', '0');
                    for (int i = 1; i < 9; i++)
                    {
                        str = str.Remove(i * 10 - i, 1);
                    }
                }
                str = str.Replace('.', '0');
                for (int i = 0; i < 81; i++)
                {
                    int c = SudokuCell.ConvertCharToCellData(str[i]);
                    if (c != 0)
                    {
                        DeleteCandidateFromThreeHouse(i / 9, i % 9, c);
                    }
                    setCellData(i / 9, i % 9, c);
                    //cells[i / 9, i % 9].Data = c;
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public string SaveToString()
        {
            string str = string.Empty;
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < m; j++)
                {
                    str += cells[i, j].Data.ToString();
                }
            }
            return str;
        }

        public string CandidateTable()
        {
            string str = string.Empty;
            str += "   |  ";
            for (int i = 0; i < m; i++)
            {
                str += "c" + (i + 1).ToString();
                for (int j = 0; j < ((i > 8) ? m - 2: m - 1); j++)
                {
                    str += " ";
                }
            }
            int l = str.Length;
            str += Environment.NewLine;
            str += "---+";
            for (int i = 0; i < l - 4; i++)
            {
                str += "-";
            }
            str += Environment.NewLine;
            for (int i = 0; i < m; i++)
            {
                str += "r" + (i + 1).ToString() + ((i > 8) ? "" : " ") + "|  ";
                for (int j = 0; j < m; j++)
                {
                    if (cells[i, j].IsData())
                    {
                        str += SudokuCell.ConvertCellDataToString(cells[i, j].Data, m);
                        for (int k = 0; k < m; k++)
                        {
                            str += " ";
                        }
                    }
                    else
                    {
                        for (int c = 1; c <= m; c++)
                        {
                            if (cells[i, j].IsCandidate(c))
                            {
                                str += SudokuCell.ConvertCellDataToString(c, m);
                            }
                        }
                        for (int k = 0; k < (m - cells[i,j].CandidatesNumber) + 1; k++)
                        {
                            str += " ";
                        }
                    }
                }
                str += Environment.NewLine;
            }
            return str;
        }

        #endregion

        #region fjlkezels

        public bool LoadFromSDXFile(string file)
        {
            try
            {
                for (int i = 0; i < m; i++)
                {
                    for (int j = 0; j < m; j++)
                    {
                        for (int c = 1; c <= m; c++)
                        {
                            cells[i, j].RemoveCandidate(c);
                        }
                    }
                }
                using (StreamReader sr = new StreamReader(file))
                {
                    filename = file;
                    string sor;
                    int i = 0;
                    while ((sor = sr.ReadLine()) != null)
                    {
                        int j = 0;
                        string[] st = sor.Split(' ');
                        for (int k = 0; k < st.Length; k++)
                        {

                            if (st[k][0] == '{' && st[k][st[k].Length - 1] == '}')
                            {
                                //ha jelltek vannak {} levlasztsa
                                st[k] = st[k].Substring(1, st[k].Length - 2);
                                for (int l = 0; l < st[k].Length; l++)
                                {
                                    cells[i, j].AddCandidate(Convert.ToInt32(st[k][l].ToString()));
                                }
                                j++;
                            }
                            else
                            {
                                if (st[k][0] == 'u')
                                {
                                    //cells[i, j].Data = Convert.ToInt32(st[k][1].ToString());
                                    setCellData(i, j, Convert.ToInt32(st[k][1].ToString()));
                                    cells[i, j++].Foundby = -1;
                                }
                                else
                                {
                                    if (st[k].Length == 1)
                                    {
                                        //cells[i, j++].Data = Convert.ToInt32(st[k].ToString());
                                        setCellData(i, j++, Convert.ToInt32(st[k].ToString()));
                                    }
                                    else
                                    {
                                        for (int l = 0; l < st[k].Length; l++)
                                        {
                                            cells[i, j].AddCandidate(Convert.ToInt32(st[k][l].ToString()));
                                        }
                                        j++;
                                    }
                                }
                            }
                        }
                        i++;
                    }
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool LoadFromSDXXFile(string file)
        {
            try
            {
                using (StreamReader sr = new StreamReader(file))
                {
                    filename = file;
                    string sor;
                    int i = 0;
                    while ((sor = sr.ReadLine()) != null)
                    {
                        int j = 0;
                        string[] st = sor.Split(' ');
                        if (i == 0)
                        {
                            N = (int)Math.Sqrt(st.Length);
                            SudokuTableInit();
                            for (int ii = 0; ii < m; ii++)
                            {
                                for (int jj = 0; jj < m; jj++)
                                {
                                    for (int c = 1; c <= m; c++)
                                    {
                                        cells[ii, jj].RemoveCandidate(c);
                                    }
                                }
                            }
                        }
                        for (int k = 0; k < st.Length; k++)
                        {
                            if (st[k][0] == 'u')
                            {
                                setCellData(i, j, Convert.ToInt32(st[k].Substring(1, st[k].Length - 1)));
                                cells[i, j++].Foundby = -1;
                            }
                            else
                            {
                                if (st[k][0] == '{' && st[k][st[k].Length - 1] == '}')
                                {
                                    st[k] = st[k].Substring(1, st[k].Length - 2);
                                    if (st[k] != string.Empty)
                                    {
                                        string[] st2 = st[k].Split(',');
                                        for (int l = 0; l < st2.Length; l++)
                                        {
                                            cells[i, j].AddCandidate(Convert.ToInt32(st2[l].ToString()));
                                        }
                                    }
                                }
                                else
                                {
                                    setCellData(i, j, Convert.ToInt32(st[k].ToString()));
                                }
                                j++;
                            }
                        }
                        i++;
                    }
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool LoadFromSDKFile(string file)
        {
            try
            {
                using (StreamReader sr = new StreamReader(file))
                {
                    filename = file;
                    string sor;
                    int i = 0;
                    while ((sor = sr.ReadLine()) != null)
                    {
                        if (sor[0] == '#')
                        {
                            switch (sor[1])
                            {
                                case 'A':   //Author
                                    properties[0] = sor.Substring(2);
                                    break;
                                case 'D':   //Description
                                    properties[1] = sor.Substring(2);
                                    break;
                                case 'C':   //Comment
                                    properties[2] = sor.Substring(2);
                                    break;
                                case 'B':   //Date published
                                    properties[3] = sor.Substring(2);
                                    break;
                                case 'S':   //Source (name of newspaper, book or website)
                                    properties[4] = sor.Substring(2);
                                    break;
                                case 'L':   //Level
                                    properties[5] = sor.Substring(2);
                                    break;
                                case 'V':   //Variant
                                    properties[6] = sor.Substring(2);
                                    break;
                                case 'U':   //Source URL
                                    properties[7] = sor.Substring(2);
                                    break;
                            }
                        }
                        else
                        {
                            int l = sor.Length;
                            int j = 0;
                            while (j < l)
                            {
                                if (sor[j] != '.')
                                {
                                    int c = Convert.ToInt32(sor[j].ToString());
                                    //cells[i, j].Data = c;
                                    setCellData(i, j, c);
                                    DeleteCandidateFromThreeHouse(i, j, c);
                                }
                                else
                                {
                                    //cells[i, j].Data = 0;
                                    setCellData(i, j, 0);
                                }
                                j++;
                            }
                            i++;
                        }
                    }
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool SaveToSDKFile()
        {
            return SaveToSDKFile(filename);
        }
        
        public bool SaveToSDKFile(string file)
        {
            if (n != 3)
            {
                System.Windows.Forms.MessageBox.Show("Az SDK fjlformtum csak 9x9-es sudoku-t tmogat");
                return false;
            }
            try
            {
                using (StreamWriter sw = new StreamWriter(file))
                {
                    for (int i = 0; i < 8; i++)
                    {
                        if (properties[i] != null)
                        {
                            switch (i)
                            { 
                                case 0:
                                    sw.WriteLine("#A" + properties[i]);
                                    break;
                                case 1:
                                    sw.WriteLine("#D" + properties[i]);
                                    break;
                                case 2:
                                    sw.WriteLine("#C" + properties[i]);
                                    break;
                                case 3:
                                    sw.WriteLine("#B" + properties[i]);
                                    break;
                                case 4:
                                    sw.WriteLine("#S" + properties[i]);
                                    break;
                                case 5:
                                    sw.WriteLine("#L" + properties[i]);
                                    break;
                                case 6:
                                    sw.WriteLine("#V" + properties[i]);
                                    break;
                                case 7:
                                    sw.WriteLine("#U" + properties[i]);
                                    break;
                            }
                        
                        }
                        
                    }
                    for (int i = 0; i < m; i++)
                    {
                        for (int j = 0; j < m; j++)
                        {
                            sw.Write(((cells[i, j].IsData()) ? cells[i, j].Data.ToString() : "."));
                        }
                        if (i != m - 1) sw.Write(sw.NewLine);
                    }
                }
                filename = file;
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool SaveToSDXFile()
        {
            return SaveToSDXFile(filename);
        }

        public bool SaveToSDXFile(string file)
        {
            if (n != 3)
            {
                System.Windows.Forms.MessageBox.Show("Az SDX fjlformtum csak 9x9-es sudoku-t tmogat");
                return false;
            }
            try
            {
                using (StreamWriter sw = new StreamWriter(file))
                {
                    for (int i = 0; i < m; i++)
                    {
                        for (int j = 0; j < m; j++)
                        {
                            if (cells[i, j].IsData())
                            {
                                sw.Write(((cells[i, j].Foundby != 0) ? "u" : "") + cells[i, j].Data.ToString());
                            }
                            else
                            {
                                sw.Write("{");
                                for (int c = 1; c <= m; c++)
                                { 
                                    if (cells[i,j].IsCandidate(c)) 
                                    {
                                        sw.Write(c.ToString());
                                    }
                                }
                                sw.Write("}");
                            }
                            if (j != m - 1) sw.Write(" ");
                        }
                        if (i != m - 1) sw.Write(sw.NewLine);
                    }
                }
                filename = file;
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool SaveToSDXXFile()
        {
            return SaveToSDXXFile(filename);
        }

        public bool SaveToSDXXFile(string file)
        {
            try
            {
                using (StreamWriter sw = new StreamWriter(file))
                {
                    for (int i = 0; i < m; i++)
                    {
                        for (int j = 0; j < m; j++)
                        {
                            if (cells[i, j].IsData())
                            {
                                sw.Write(/*((cells[i, j].Foundby != 0) ? "u" : "") + */cells[i, j].Data.ToString());
                            }
                            else
                            {
                                sw.Write("{");
                                int k = 1;
                                for (int c = 1; c <= m; c++)
                                {
                                    if (cells[i, j].IsCandidate(c))
                                    {
                                        sw.Write(c.ToString() + ((k++ < cells[i, j].CandidatesNumber) ? "," : ""));
                                    }
                                }
                                sw.Write("}");
                            }
                            if (j != m - 1) sw.Write(" ");
                        }
                        if (i != m - 1) sw.Write(sw.NewLine);
                    }
                }
                filename = file;
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        #endregion

        #region ICloneable Members (IClonable interfsz megvalstsa)

        public object Clone()
        {
            SudokuTable t = new SudokuTable(n);
            t.m = m;
            t.n = n;
            t.filename = filename;
            t.data_in_table = data_in_table;
            for (int i = 0; i < m; i++)
            {
                for (int j = 0; j < m; j++)
                {
                    t.cells[i, j] = cells[i, j].Clone() as SudokuCell;
                }
            }
            for (int i = 0; i < propnum; i++)
            {
                t.properties[i] = properties[i];
            }
            return t;
        }

        #endregion
    }
}
